#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.common.skin.breadcrumbs module

This module defines components used for shared contents breadcrumbs management.
"""

from zope.interface import Interface
from zope.location.interfaces import ILocation

from pyams_content.shared.common.interfaces.types import IWfTypedSharedContent
from pyams_layer.interfaces import IPyAMSUserLayer
from pyams_skin.interfaces.viewlet import IBreadcrumbs
from pyams_skin.viewlet.breadcrumb import LocationBreadcrumbs as BaseLocationBreadcrumbs
from pyams_template.template import override_template
from pyams_utils.adapter import adapter_config
from pyams_utils.interfaces import DISPLAY_CONTEXT_KEY_NAME

__docformat__ = 'restructuredtext'


@adapter_config(required=(ILocation, IPyAMSUserLayer, Interface),
                provides=IBreadcrumbs)
class BreadcrumbsAdapter(BaseLocationBreadcrumbs):
    """Breadcrumbs adapter"""

    @property
    def items(self):
        source = self.request.annotations.get(DISPLAY_CONTEXT_KEY_NAME)
        if source is None:
            source = self.request.context
        if source is not None:
            yield from self.get_items(source)


override_template(BreadcrumbsAdapter,
                  template='templates/breadcrumbs.pt', layer=IPyAMSUserLayer)


@adapter_config(required=(IWfTypedSharedContent, IPyAMSUserLayer, Interface),
                provides=IBreadcrumbs)
class TypedSharedContentBreadcrumbsAdapter(BreadcrumbsAdapter):
    """Typed shared content breadcrumbs adapter"""

    @property
    def items(self):
        data_type = self.context.get_data_type()
        if data_type is not None:
            source = data_type.get_source_folder()
            if source is not None:
                yield from self.get_items(source)
                return
        yield from super(TypedSharedContentBreadcrumbsAdapter, self).items



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.view.interfaces.query module

This module defines views queries interfaces.
"""

from enum import Enum

from zope.interface import Attribute, Interface

__docformat__ = 'restructuredtext'


END_PARAMS_MARKER = object()


class IViewQuery(Interface):
    """View query interface"""

    def get_params(self, context, request=None, **kwargs):
        """Get static view query params"""

    def get_results(self, context, sort_index, reverse, limit,
                    request=None, aggregates=None, settings=None, **kwargs):
        """Get tuple of limited results and total results count"""


class IViewUserQuery(Interface):
    """View user search query interface"""

    def get_user_params(self, request):
        """Get dynamic user query params"""


class IViewQueryExtension(Interface):
    """Base view query extension"""

    weight = Attribute("Extension weight")


class IViewQueryParamsExtension(IViewQueryExtension):
    """View query extension interface"""

    def get_params(self, context, request=None):
        """Add params to catalog query

        This method may return an iterator.
        If defined settings are such that no result can be found (for example, if the view is
        defined to use context themes but context doesn't have any), method can yield a "None"
        value which will cancel query execution.
        """


class IViewQueryFilterExtension(IViewQueryExtension):
    """View query filter extension

    Query filters are applied after query execution, so they are generally slower than
    normal queries, but can sometimes handle more complex rules.
    """

    def filter(self, context, items, request=None):
        """Filter items after catalog query"""


class MergeModes(Enum):
    """Merge modes enumeration"""
    CONCAT = 'concat'
    RANDOM = 'random'
    ZIP = 'zip'
    RANDOM_ZIP = 'zip_random'


VIEWS_MERGERS_VOCABULARY = 'pyams_content.views.mergers'


class IViewsMerger(Interface):
    """Interface used to define views mergers

    Mergers are used to merge results of several views.
    """

    def get_results(self, views, context, ignore_cache=False, request=None,
                    aggregates=None, settings=None):
        """Merge results of several views together"""


EXCLUDED_VIEW_ITEMS = 'pyams_content.view.excluded'



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.cards.zmi module

This module defines components which are used for management interface
of Bootstrap cards paragraphs.
"""

from pyams_content.component.cards.interfaces import CARDS_PARAGRAPH_ICON_CLASS, CARDS_PARAGRAPH_NAME, \
    CARDS_PARAGRAPH_TYPE, ICardsParagraph
from pyams_content.component.paragraph.interfaces import IParagraphContainer, IParagraphContainerTarget
from pyams_content.component.paragraph.zmi import BaseParagraphAddForm, BaseParagraphAddMenu
from pyams_content.component.paragraph.zmi.interfaces import IParagraphContainerBaseTable
from pyams_form.ajax import ajax_form_config
from pyams_layer.interfaces import IPyAMSLayer
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IContextAddingsViewletManager

__docformat__ = 'restructuredtext'


@viewlet_config(name='add-cards-paragraph.menu',
                context=IParagraphContainerTarget, layer=IAdminLayer,
                view=IParagraphContainerBaseTable,
                manager=IContextAddingsViewletManager, weight=800)
class CardsParagraphAddMenu(BaseParagraphAddMenu):
    """Cards paragraph add menu"""

    label = CARDS_PARAGRAPH_NAME
    icon_class = CARDS_PARAGRAPH_ICON_CLASS

    factory_name = CARDS_PARAGRAPH_TYPE
    href = 'add-cards-paragraph.html'


@ajax_form_config(name='add-cards-paragraph.html',
                  context=IParagraphContainer, layer=IPyAMSLayer)
class CardsParagraphAddForm(BaseParagraphAddForm):
    """Cards paragraph add form"""

    content_factory = ICardsParagraph



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.reference.pictogram module

"""

__docformat__ = 'restructuredtext'

from pyramid.events import subscriber
from zope.component.interfaces import ISite
from zope.lifecycleevent import IObjectAddedEvent
from zope.schema.fieldproperty import FieldProperty

from pyams_content.reference import ReferenceInfo, ReferenceTable, ReferencesVocabulary
from pyams_content.reference.pictogram.interfaces import IPictogram, IPictogramTable, \
    PICTOGRAM_VOCABULARY
from pyams_file.property import I18nFileProperty
from pyams_utils.factory import factory_config
from pyams_utils.traversing import get_parent
from pyams_utils.vocabulary import vocabulary_config


@factory_config(IPictogramTable)
class PictogramTable(ReferenceTable):
    """Pictogram table"""


@subscriber(IObjectAddedEvent, context_selector=IPictogramTable)
def handle_added_pictogram_table(event):
    """Handle new pictogram table"""
    site = get_parent(event.object, ISite)
    registry = site.getSiteManager()
    if registry is not None:
        registry.registerUtility(event.object, IPictogramTable)


@factory_config(IPictogram)
class Pictogram(ReferenceInfo):
    """Pictogram persistent class"""

    image = I18nFileProperty(IPictogram['image'])
    alt_title = FieldProperty(IPictogram['alt_title'])
    header = FieldProperty(IPictogram['header'])


@vocabulary_config(name=PICTOGRAM_VOCABULARY)
class PictogramsVocabulary(ReferencesVocabulary):
    """Pictograms vocabulary"""

    table_interface = IPictogramTable



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.skin.interfaces module

this module defines common contents rendering interfaces.
"""

__docformat__ = 'restructuredtext'

from zope.interface import Attribute, Interface

from pyams_layer.interfaces import IPyAMSUserLayer


class IPyAMSDefaultLayer(IPyAMSUserLayer):
    """PyAMS default skin layer interface"""


class IContentTitle(Interface):
    """Content title getter interface"""


class IContentIllustration(Interface):
    """Content illustration getter interface"""


class IContentBannerIllustration(Interface):
    """Content banner illustration getter interface"""


class IContentNavigationIllustration(Interface):
    """Content navigation illustration getter interface"""


class IContentNavigationTitle(Interface):
    """Content navigation title getter interface"""


class IContentTag(Interface):
    """Shared content data type tag getter interface"""


class IContentDate(Interface):
    """Content date getter interface"""


class IContentHeader(Interface):
    """Content header getter interface"""


class IContentSummaryInfo(Interface):
    """Content summary info getter interface"""

    context = Attribute("Link to adapted context")
    title = Attribute("Content title")
    header = Attribute("Content header")
    button_title = Attribute("Button title")


class IPublicURL(Interface):
    """Public URL target getter interface"""


class ISearchResultsView(Interface):
    """Search results view marker interface"""


class ISearchResultsPanelsView(ISearchResultsView):
    """Search results panels view marker interface"""


class ISearchResultsCardsView(ISearchResultsView):
    """Search results cards view marker interface"""



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.view.portlet.interfaces module

This module defines interfaces of view items portlet settings.
"""

from collections import OrderedDict

from zope.interface import Interface
from zope.schema import Bool, Choice, Int
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from pyams_content.shared.view.interfaces import VIEW_CONTENT_TYPE
from pyams_content.shared.view.interfaces.query import MergeModes, VIEWS_MERGERS_VOCABULARY
from pyams_i18n.schema import I18nTextLineField
from pyams_portal.interfaces import IPortletSettings
from pyams_sequence.schema import InternalReferencesListField

__docformat__ = 'restructuredtext'

from pyams_content import _


#
# Views display contexts
#

VIEW_DISPLAY_CONTEXT = 'display'
VIEW_CONTENT_CONTEXT = 'content'

VIEW_CONTEXTS = OrderedDict((
    (VIEW_DISPLAY_CONTEXT, _("Display context")),
    (VIEW_CONTENT_CONTEXT, _("Content context"))
))

VIEW_CONTEXT_VOCABULARY = SimpleVocabulary([
    SimpleTerm(v, title=t)
    for v, t in VIEW_CONTEXTS.items()
])

SEARCH_EXCLUDED_ITEMS = 'search.excluded'


#
# Views merge modes
#

VIEW_ITEMS_PORTLET_NAME = 'pyams_content.portlet.view'


class IViewItemsAggregates(Interface):
    """View items aggregates interface"""


class IViewItemsPortletSettings(IPortletSettings):
    """View items portlet settings interface"""

    title = I18nTextLineField(title=_("Title"),
                              required=False)

    views = InternalReferencesListField(title=_("Selected views"),
                                        description=_("Reference to the view(s) from which items are extracted; "
                                                      "you can combine several views together and specify in which "
                                                      "order they should be mixed"),
                                        content_type=VIEW_CONTENT_TYPE,
                                        required=True)

    def get_views(self):
        """Get iterator over selected views"""

    views_context = Choice(title=_("Views context"),
                           description=_("When searching for items, a view receives a \"context\" which is the object "
                                         "from which settings can be extracted; this context can be the \"display\" "
                                         "context or the \"content\" context: when the portlet is used to display the "
                                         "site root, a site manager or a site folder, both are identical; when the "
                                         "portlet is used to display a shared content, the \"content\" context is the "
                                         "displayed content, while the \"display\" context is the container (site "
                                         "root, site manager or site folder) into which content is displayed"),
                           vocabulary=VIEW_CONTEXT_VOCABULARY,
                           default=VIEW_DISPLAY_CONTEXT,
                           required=True)

    views_merge_mode = Choice(title=_("Views merge mode"),
                              description=_("If you select several views, you can select \"merge\" mode, which is "
                                            "the way used to merge items from several views"),
                              vocabulary=VIEWS_MERGERS_VOCABULARY,
                              default=MergeModes.CONCAT.value,
                              required=True)

    def get_merger(self):
        """Get selected views merger utility"""

    limit = Int(title=_("Results count limit"),
                description=_("Maximum number of results that the component may extract from merged views"),
                required=False)

    start = Int(title=_("Starting from..."),
                description=_("You can skip several results if specifying an integer value here..."),
                required=False,
                default=1)

    def get_items(self):
        """Get iterator over items returned by selected views, using selected merger"""

    force_canonical_url = Bool(title=_("Force canonical URL?"),
                               description=_("By default, internal links use a \"relative\" URL, which tries to "
                                             "display link target in the current context; by using a canonical URL, "
                                             "you can display target in it's attachment context (if defined)"),
                               required=False,
                               default=False)

    exclude_from_search = Bool(title=_("Exclude from search results"),
                               description=_("If 'yes', and if this portlet is associated with a search engine in the "
                                             "same page template, items displayed by this portlet will be excluded "
                                             "from search results"),
                               required=True,
                               default=False)

    first_page_only = Bool(title=_("Display on first page only"),
                           description=_("If 'yes', and if this portlet is associated with a search engine in the "
                                         "same page template, view contents will only be displayed on the first page "
                                         "of search results"),
                           required=True,
                           default=False)



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.illustration.zmi module

This module provides management interface components for illustrations.
"""

from pyramid.events import subscriber

from pyams_content.component.illustration import IIllustrationTargetBase, ILinkIllustration
from pyams_content.component.illustration.interfaces import IBaseIllustration, \
    IBaseIllustrationTarget, IIllustration, IIllustrationTarget, ILinkIllustrationTarget
from pyams_content.component.paragraph.interfaces import IBaseParagraph
from pyams_zmi.interfaces.form import IPropertiesEditForm
from pyams_form.field import Fields
from pyams_form.interfaces.form import IAJAXFormRenderer, IFormContent, IFormUpdatedEvent, IInnerSubForm
from pyams_portal.zmi.portlet import PortletRendererSettingsEditForm
from pyams_portal.zmi.widget import RendererSelectFieldWidget
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_zmi.form import FormGroupSwitcher
from pyams_zmi.helper.event import get_json_widget_refresh_callback
from pyams_zmi.interfaces import IAdminLayer


__docformat__ = 'restructuredtext'

from pyams_content import _


@adapter_config(name='illustration',
                required=(IBaseIllustrationTarget, IAdminLayer, IPropertiesEditForm),
                provides=IInnerSubForm)
class BasicIllustrationPropertiesEditForm(FormGroupSwitcher):
    """Basic illustration properties edit form"""

    legend = _("Main illustration")
    weight = 10

    fields = Fields(IBaseIllustration)
    prefix = 'illustration.'

    @property
    def mode(self):
        """Form mode getter"""
        return self.parent_form.mode

    @property
    def state(self):
        """Form state getter"""
        return 'open' if self.get_content().has_data() else 'closed'


@adapter_config(required=(IIllustrationTargetBase, IAdminLayer, BasicIllustrationPropertiesEditForm),
                provides=IFormContent)
def base_illustration_edit_form_content(context, request, form):
    """Base illustration properties edit form content getter"""
    return IIllustration(context)


@adapter_config(name='illustration',
                required=(IIllustrationTarget, IAdminLayer, IPropertiesEditForm),
                provides=IInnerSubForm, force_implements=False)
class IllustrationPropertiesEditForm(BasicIllustrationPropertiesEditForm):
    """Illustration properties edit form"""

    fields = Fields(IIllustration)
    fields['renderer'].widget_factory = RendererSelectFieldWidget

    @property
    def state(self):
        """Form state getter"""
        return super().state if IBaseParagraph.providedBy(self.context) else 'open'


@adapter_config(name='link-illustration',
                required=(ILinkIllustrationTarget, IAdminLayer, IPropertiesEditForm),
                provides=IInnerSubForm, force_implements=False)
class LinkIllustrationPropertiesEditForm(BasicIllustrationPropertiesEditForm):
    """Link illustration properties edit form"""

    legend = _("Navigation link illustration")
    weight = 15

    prefix = 'link_illustration.'


@adapter_config(required=(ILinkIllustrationTarget, IAdminLayer, LinkIllustrationPropertiesEditForm),
                provides=IFormContent)
def link_illustration_edit_form_content(context, request, form):
    """Link illustration properties edit form content getter"""
    return ILinkIllustration(context)


@adapter_config(required=(IBaseIllustrationTarget, IAdminLayer,
                          BasicIllustrationPropertiesEditForm),
                provides=IAJAXFormRenderer)
class BasicIllustrationPropertiesEditFormRenderer(ContextRequestViewAdapter):
    """Basic illustration properties AJAX form renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if changes is None:
            return None
        result = {
            'status': 'success',
            'message': self.request.localizer.translate(self.view.parent_form.success_message)
        }
        if 'data' in changes.get(IBaseIllustration, ()):
            result['callbacks'] = [
                get_json_widget_refresh_callback(self.view, 'data', self.request)
            ]
        return result


@subscriber(IFormUpdatedEvent,
            context_selector=IBaseIllustration,
            form_selector=PortletRendererSettingsEditForm)
def handle_illustration_renderer_settings_edit_form_update(event):
    """Illustration renderer settings edit form update"""
    widgets = event.form.widgets
    thumb_selection = widgets.get('thumb_selection')
    if thumb_selection is not None:
        thumb_selection.no_value_message = _("Use responsive selection")



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.common.skin.specificities module

This module defines a custom content provider used for shared contents specificities
rendering.
"""

from zope.interface import Interface

from pyams_content.shared.common.portlet.interfaces import ISpecificitiesRenderer
from pyams_layer.interfaces import IPyAMSUserLayer
from pyams_viewlet.viewlet import ViewContentProvider, contentprovider_config

__docformat__ = 'restructuredtext'


@contentprovider_config(name='pyams_content.specificities',
                        layer=IPyAMSUserLayer, view=Interface)
class SharedContentSpecificitiesContentProvider(ViewContentProvider):
    """Shared content specificities content provider"""

    renderer = None

    def update(self, name=''):
        super().update()
        registry = self.request.registry
        renderer = registry.queryMultiAdapter((self.context, self.request, self.view),
                                              ISpecificitiesRenderer,
                                              name=name)
        if renderer is not None:
            renderer.update()
            self.renderer = renderer

    def render(self, template_name=''):
        if self.renderer is None:
            return ''
        return self.renderer.render(template_name)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.video.skin.provider.dailymotion module

This module defines Dailymotion video renderer.
"""

from pyams_content.component.video.provider.interfaces import IDailymotionVideoSettings
from pyams_content.component.video.skin import IExternalVideoRenderer
from pyams_content.component.video.skin.provider import BaseExternalVideoRenderer, time_to_seconds
from pyams_layer.interfaces import IPyAMSLayer
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config

__docformat__ = 'restructuredtext'


DAILYMOTION_PARAMS = (
    ('start_at', 'start', time_to_seconds),
    ('autoplay', 'autoplay', int),
    ('show_info', 'ui-start-screen-info', int),
    ('show_commands', 'controls', int),
    ('ui_theme', 'ui-theme', str),
    ('show_branding', 'ui-logo', int),
    ('show_endscreen', 'endscreen-enable', int),
    ('allow_sharing', 'sharing-enable', int)
)


@adapter_config(required=(IDailymotionVideoSettings, IPyAMSLayer),
                provides=IExternalVideoRenderer)
@template_config(template='templates/dailymotion-render.pt', layer=IPyAMSLayer)
class DailymotionVideoRenderer(BaseExternalVideoRenderer):
    """Dailymotion video renderer"""

    params = DAILYMOTION_PARAMS



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyramid.decorator import reify
from zope.interface import Interface

from pyams_content.reference.pictogram import IPictogramTable
from pyams_content.reference.zmi.table import ReferenceTableContainerTable
from pyams_file.skin import render_image
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_table.interfaces import IColumn
from pyams_utils.adapter import adapter_config
from pyams_utils.interfaces import MISSING_INFO
from pyams_viewlet.manager import viewletmanager_config
from pyams_zmi.interfaces import IAdminLayer, IObjectLabel
from pyams_zmi.interfaces.form import IFormTitle, IPropertiesEditForm
from pyams_zmi.interfaces.viewlet import IPropertiesMenu, ISiteManagementMenu
from pyams_zmi.table import ActionColumn, TableAdminView
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem

__docformat__ = 'restructuredtext'

from pyams_content import _


PICTOGRAM_TABLE_LABEL = _("Pictograms")


@adapter_config(required=(IPictogramTable, IPyAMSLayer, Interface),
                provides=IObjectLabel)
def pictogram_table_label(context, request, view):
    """Pictograms table label"""
    return request.localizer.translate(PICTOGRAM_TABLE_LABEL)


@viewletmanager_config(name='contents.menu',
                       context=IPictogramTable, layer=IAdminLayer,
                       manager=ISiteManagementMenu, weight=10,
                       permission=VIEW_SYSTEM_PERMISSION,
                       provides=IPropertiesMenu)
class PictogramTableContentsMenu(NavigationMenuItem):
    """Pictogram table contents menu"""

    label = _("Table contents")
    icon_class = 'fas fa-table'
    href = '#contents.html'


class PictogramTableContainerTable(ReferenceTableContainerTable):
    """Pictogram container table"""

    @reify
    def data_attributes(self):
        attributes = super().data_attributes
        attributes['table'].update({
            'data-ams-order': '1,asc'
        })
        return attributes


@adapter_config(name='image',
                required=(IPictogramTable, IAdminLayer, PictogramTableContainerTable),
                provides=IColumn)
class PictogramTableImageColumn(ActionColumn):
    """Pictogram table image column"""

    css_classes = {
        'th': 'action',
        'td': 'action p-1'
    }
    weight = 5

    def render_cell(self, item):
        image = II18n(item).query_attribute('image', request=self.request)
        if image:
            return render_image(image, 32, 32, self.request, timestamp=True)
        return MISSING_INFO


@pagelet_config(name='contents.html',
                context=IPictogramTable, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class PictogramTableContentsView(TableAdminView):
    """Pictograms table contents view"""

    title = _("Pictograms")

    table_class = PictogramTableContainerTable
    table_label = _("Pictograms list")


@adapter_config(required=(IPictogramTable, IAdminLayer, IPropertiesEditForm),
                provides=IFormTitle)
def pictograms_table_title(context, request, form):
    """Pictograms table edit form title getter"""
    translate = request.localizer.translate
    return translate(_("Pictograms table"))



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.alert.zmi.types module

"""

from pyramid.view import view_config
from zope.interface import Interface

from pyams_content.interfaces import MANAGE_TOOL_PERMISSION
from pyams_content.shared.alert import IAlertManager, IAlertTypesManager
from pyams_content.shared.alert.interfaces import IAlertType
from pyams_content.shared.alert.zmi.interfaces import IAlertTypesTable
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IAJAXFormRenderer
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.interfaces.view import IModalEditForm
from pyams_skin.viewlet.actions import ContextAddAction
from pyams_table.interfaces import IColumn, IValues
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.factory import factory_config
from pyams_utils.interfaces.intids import IUniqueID
from pyams_utils.traversing import get_parent
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminModalAddForm, AdminModalEditForm
from pyams_zmi.helper.container import delete_container_element, switch_element_attribute
from pyams_zmi.helper.event import get_json_table_row_add_callback, get_json_table_row_refresh_callback
from pyams_zmi.interfaces import IAdminLayer, IObjectHint, IObjectLabel, TITLE_SPAN_BREAK
from pyams_zmi.interfaces.form import IFormTitle
from pyams_zmi.interfaces.table import ITableElementEditor
from pyams_zmi.interfaces.viewlet import IPropertiesMenu, IToolbarViewletManager
from pyams_zmi.table import NameColumn, ReorderColumn, SortableTable, TableAdminView, TableElementEditor, TrashColumn, \
    VisibilityColumn
from pyams_zmi.utils import get_object_hint, get_object_label
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem

__docformat__ = 'restructuredtext'

from pyams_content import _


@viewlet_config(name='alert-types.menu',
                context=IAlertManager, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=400,
                permission=MANAGE_TOOL_PERMISSION)
class AlertTypesMenu(NavigationMenuItem):
    """Alert types menu"""

    label = _("Alert types")
    href = '#alert-types.html'


@factory_config(IAlertTypesTable)
class AlertTypesTable(SortableTable):
    """Alert types table"""

    container_class = IAlertTypesManager

    display_if_empty = True


@adapter_config(required=(IAlertManager, IAdminLayer, IAlertTypesTable),
                provides=IValues)
class AlertTypesTableValues(ContextRequestViewAdapter):
    """Alert types table values"""

    @property
    def values(self):
        """Alert types values getter"""
        yield from IAlertTypesManager(self.context).values()


@adapter_config(name='reorder',
                required=(IAlertManager, IAdminLayer, IAlertTypesTable),
                provides=IColumn)
class AlertTypesReorderColumn(ReorderColumn):
    """Alert types table reorder column"""


@view_config(name='reorder.json',
             context=IAlertTypesManager, request_type=IPyAMSLayer,
             renderer='json', xhr=True,
             permission=MANAGE_TOOL_PERMISSION)
def reorder_types_table(request):
    """Reorder alert types"""
    order = request.params.get('order').split(';')
    request.context.updateOrder(order)
    return {
        'status': 'success'
    }


@adapter_config(name='visible',
                required=(IAlertManager, IAdminLayer, IAlertTypesTable),
                provides=IColumn)
class AlertTypesVisibleColumn(VisibilityColumn):
    """Alert types table visible column"""

    hint = _("Click icon to enable or disable alert type")


@view_config(name='switch-visible-item.json',
             context=IAlertTypesManager, request_type=IPyAMSLayer,
             renderer='json', xhr=True)
def switch_visible_item(request):
    """Switch visible item"""
    return switch_element_attribute(request)


@adapter_config(name='label',
                required=(IAlertManager, IAdminLayer, IAlertTypesTable),
                provides=IColumn)
class AlertTypesLabelColumn(NameColumn):
    """Alert types table label column"""

    i18n_header = _("Label")


@adapter_config(name='trash',
                required=(IAlertManager, IAdminLayer, IAlertTypesTable),
                provides=IColumn)
class AlertTypesTrashColumn(TrashColumn):
    """Alert types table trash column"""


@view_config(name='delete-element.json',
             context=IAlertTypesManager, request_type=IPyAMSLayer,
             renderer='json', xhr=True,
             permission=MANAGE_TOOL_PERMISSION)
def delete_data_type(request):
    """Delete data type"""
    return delete_container_element(request)


@pagelet_config(name='alert-types.html',
                context=IAlertManager, layer=IPyAMSLayer,
                permission=MANAGE_TOOL_PERMISSION)
class AlertTypesView(TableAdminView):
    """Alert types view"""

    title = _("Alert types")

    table_class = IAlertTypesTable
    table_label = _("Alert types list")
    
    
#
# Alert types views
#

@viewlet_config(name='add-alert-type.action',
                context=IAlertManager, layer=IAdminLayer, view=IAlertTypesTable,
                manager=IToolbarViewletManager, weight=20,
                permission=MANAGE_TOOL_PERMISSION)
class AlertTypesAddAction(ContextAddAction):
    """Alert type add action"""

    label = _("Add alert type")
    href = 'add-alert-type.html'


@ajax_form_config(name='add-alert-type.html',
                  context=IAlertManager, layer=IPyAMSLayer,
                  permission=MANAGE_TOOL_PERMISSION)
class AlertTypeAddForm(AdminModalAddForm):
    """Alert type add form"""

    subtitle = _("New alert type")
    legend = _("New alert type properties")

    fields = Fields(IAlertType).omit('__name__', '__parent__', 'visible')
    content_factory = IAlertType

    def add(self, obj):
        oid = IUniqueID(obj).oid
        IAlertTypesManager(self.context)[oid] = obj


@adapter_config(required=(IAlertManager, IAdminLayer, AlertTypeAddForm),
                provides=IAJAXFormRenderer)
class AlertTypeAddFormRenderer(ContextRequestViewAdapter):
    """Alert type add form renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if changes is None:
            return None
        return {
            'status': 'success',
            'callbacks': [
                get_json_table_row_add_callback(self.context, self.request,
                                                IAlertTypesTable, changes)
            ]
        }


@adapter_config(required=(IAlertType, IAdminLayer, Interface),
                provides=IObjectLabel)
def alert_type_label(context, request, view):
    """Alert type label"""
    i18n = II18n(context)
    return i18n.query_attribute('label', request=request)


@adapter_config(required=(IAlertType, IAdminLayer, Interface),
                provides=IObjectHint)
def alert_type_hint(context, request, view):  # pylint: disable=unused-argument
    """Alert type hint"""
    return request.localizer.translate(_("Alert type"))


@adapter_config(required=(IAlertType, IAdminLayer, IAlertTypesTable),
                provides=ITableElementEditor)
class AlertTypeEditor(TableElementEditor):
    """Alert type editor"""


@ajax_form_config(name='properties.html',
                  context=IAlertType, layer=IPyAMSLayer,
                  permission=MANAGE_TOOL_PERMISSION)
class AlertTypeEditForm(AdminModalEditForm):
    """Alert type properties edit form"""

    legend = _("Alert type properties")

    fields = Fields(IAlertType).omit('__name__', '__parent__', 'visible')


@adapter_config(required=(IAlertType, IAdminLayer, IModalEditForm),
                provides=IFormTitle)
def alert_type_edit_form_title(context, request, form):
    """Alert type edit form title getter"""
    return TITLE_SPAN_BREAK.format(
        get_object_hint(context, request, form),
        get_object_label(context, request, form))


@adapter_config(required=(IAlertType, IAdminLayer, AlertTypeEditForm),
                provides=IAJAXFormRenderer)
class AlertTypeEditFormRenderer(ContextRequestViewAdapter):
    """Alert type edit form AJAX renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if not changes:
            return None
        tool = get_parent(self.context, IAlertManager)
        return {
            'callbacks': [
                get_json_table_row_refresh_callback(tool, self.request,
                                                    IAlertTypesTable, self.context)
            ]
        }



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.fields.paragraph module

This module provides form fields paragraph persistent classes and adapters.
"""

from zope.schema.fieldproperty import FieldProperty
from pyams_content.component.fields.interfaces import FORM_FIELDS_PARAGRAPH_ICON_CLASS, \
    FORM_FIELDS_PARAGRAPH_NAME, FORM_FIELDS_PARAGRAPH_RENDERERS, FORM_FIELDS_PARAGRAPH_TYPE, \
    IFormFieldsParagraph
from pyams_content.component.paragraph import BaseParagraph, IBaseParagraph
from pyams_content.feature.renderer import RenderersVocabulary
from pyams_utils.factory import factory_config


__docformat__ = 'restructuredtext'

from pyams_utils.vocabulary import vocabulary_config


@factory_config(IFormFieldsParagraph)
@factory_config(IBaseParagraph, name=FORM_FIELDS_PARAGRAPH_TYPE)
class FormFieldsParagraph(BaseParagraph):
    """Form fields paragraph"""

    factory_name = FORM_FIELDS_PARAGRAPH_TYPE
    factory_label = FORM_FIELDS_PARAGRAPH_NAME
    factory_intf = IFormFieldsParagraph

    icon_class = FORM_FIELDS_PARAGRAPH_ICON_CLASS

    renderer = FieldProperty(IFormFieldsParagraph['renderer'])


@vocabulary_config(name=FORM_FIELDS_PARAGRAPH_RENDERERS)
class FormFieldsParagraphRenderersVocabulary(RenderersVocabulary):
    """Form fields paragraph renderers vocabulary"""

    content_interface = IFormFieldsParagraph



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.association.container module

This module provides associations container component.
"""

__docformat__ = 'restructuredtext'

from zope.container.ordered import OrderedContainer
from zope.location import locate
from zope.location.interfaces import ISublocations
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary
from zope.traversing.interfaces import ITraversable

from pyams_catalog.utils import index_object
from pyams_content.component.association import IAssociationContainer, \
    IAssociationContainerTarget, IAssociationItem
from pyams_content.component.association.interfaces import ASSOCIATION_CONTAINER_KEY, \
    ASSOCIATION_VOCABULARY, IAssociationInfo
from pyams_utils.adapter import ContextAdapter, adapter_config, get_annotation_adapter
from pyams_utils.factory import factory_config
from pyams_utils.registry import get_pyramid_registry
from pyams_utils.traversing import get_parent
from pyams_utils.vocabulary import vocabulary_config
from pyams_zmi.interfaces import IAdminLayer


@factory_config(IAssociationContainer)
class AssociationContainer(OrderedContainer):
    """Association container"""

    last_id = 1

    def append(self, value, notify=True):
        """Append item to container"""
        key = str(self.last_id)
        if not notify:
            # pre-locate association item to avoid multiple notifications
            locate(value, self, key)
        self[key] = value
        self.last_id += 1
        if not notify:
            # make sure that association item is correctly indexed
            index_object(value)

    def get_visible_items(self, request=None):
        """Visible items iterator"""
        for item in filter(lambda x: IAssociationItem(x).visible, self.values()):
            if IAdminLayer.providedBy(request) or item.is_visible(request):
                yield item


@adapter_config(required=IAssociationContainerTarget,
                provides=IAssociationContainer)
def association_container_factory(target):
    """Associations container factory"""
    return get_annotation_adapter(target, ASSOCIATION_CONTAINER_KEY, IAssociationContainer,
                                  name='++ass++')


@adapter_config(name='ass',
                required=IAssociationContainerTarget,
                provides=ITraversable)
class AssociationContainerNamespace(ContextAdapter):
    """Associations container ++ass++ namespace"""

    def traverse(self, name, furtherpath=None):  # pylint: disable=unused-argument
        """Associations container traverser"""
        registry = get_pyramid_registry()
        return registry.queryAdapter(self.context, IAssociationContainer, name=name or '')


@adapter_config(name='associations',
                required=IAssociationContainerTarget,
                provides=ISublocations)
class AssociationContainerSublocations(ContextAdapter):
    """Associations container sub-locations adapter"""

    def sublocations(self):
        """Sub-locations iterator"""
        yield from IAssociationContainer(self.context).values()


@vocabulary_config(name=ASSOCIATION_VOCABULARY)
class ContentAssociationsVocabulary(SimpleVocabulary):
    """Content associations vocabulary"""

    def __init__(self, context=None):
        terms = []
        target = get_parent(context, IAssociationContainerTarget)
        if target is not None:
            terms = [
                SimpleTerm(link.__name__, title=IAssociationInfo(link).inner_title)
                for link in IAssociationContainer(target).values()
            ]
        super().__init__(terms)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.view module

This module defines views persistent classes.
"""

import logging
from itertools import islice, tee

from pyramid.events import subscriber
from zope.interface import implementer
from zope.intid.interfaces import IIntIds
from zope.lifecycleevent.interfaces import IObjectModifiedEvent
from zope.schema.fieldproperty import FieldProperty

from pyams_catalog.query import CatalogResultSet
from pyams_content.feature.preview.interfaces import IPreviewTarget
from pyams_content.shared.common import ISharedContent, IWfSharedContent, SharedContent, WfSharedContent
from pyams_content.shared.common.interfaces.types import IWfTypedSharedContent
from pyams_content.shared.view.interfaces import IView, IWfView, VIEW_CONTENT_NAME, VIEW_CONTENT_TYPE
from pyams_content.shared.view.interfaces.query import IViewQuery
from pyams_content.shared.view.interfaces.settings import IViewSettings
from pyams_utils.cache import get_cache
from pyams_utils.factory import factory_config, get_all_factories
from pyams_utils.interfaces import ICacheKeyValue
from pyams_utils.registry import get_pyramid_registry, get_utility
from pyams_utils.request import check_request


LOGGER = logging.getLogger('PyAMS (content)')

VIEWS_CACHE_NAME = 'views'
VIEWS_CACHE_REGION = 'views'
VIEWS_CACHE_NAMESPACE = 'PyAMS::views'

VIEW_CACHE_KEY = 'view::{view}'
VIEW_CONTEXT_CACHE_KEY = 'view::{view}.context::{context}'

_MARKER = object()


@factory_config(IWfView)
@factory_config(IWfSharedContent, name=VIEW_CONTENT_TYPE)
@implementer(IPreviewTarget)
class WfView(WfSharedContent):
    """View persistent class"""

    content_type = VIEW_CONTENT_TYPE
    content_name = VIEW_CONTENT_NAME
    content_intf = IWfView
    content_view = False

    handle_content_url = False
    handle_header = False
    handle_description = False

    select_context_path = FieldProperty(IWfView['select_context_path'])
    select_context_type = FieldProperty(IWfView['select_context_type'])
    selected_content_types = FieldProperty(IWfView['selected_content_types'])
    select_context_datatype = FieldProperty(IWfView['select_context_datatype'])
    selected_datatypes = FieldProperty(IWfView['selected_datatypes'])
    excluded_content_types = FieldProperty(IWfView['excluded_content_types'])
    excluded_datatypes = FieldProperty(IWfView['excluded_datatypes'])
    allow_user_params = FieldProperty(IWfView['allow_user_params'])
    order_by = FieldProperty(IWfView['order_by'])
    reversed_order = FieldProperty(IWfView['reversed_order'])
    limit = FieldProperty(IWfView['limit'])
    age_limit = FieldProperty(IWfView['age_limit'])

    @property
    def is_using_context(self):
        """Check to know if view is using any context properties"""
        if self.select_context_path or self.select_context_type:
            return True
        registry = get_pyramid_registry()
        for name, adapter in registry.getAdapters((self,), IViewSettings):
            if not name:
                continue
            if adapter.is_using_context:
                return True
        return False

    def get_content_path(self, context):
        """Get path value of provided context"""
        if self.select_context_path:
            intids = get_utility(IIntIds)
            return intids.queryId(context)

    @staticmethod
    def get_ignored_types():
        """Get content types ignored from views, except if specified explicitly at runtime"""
        yield from (
            content_type
            for content_type, factory in get_all_factories(IWfSharedContent)
            if not factory.content_view
        )

    def get_content_types(self, context):
        """Get selected content types"""
        content_types = set()
        if self.select_context_type and IWfSharedContent.providedBy(context):
            content_types.add(context.content_type)
        if self.selected_content_types:
            content_types |= set(self.selected_content_types)
        return list(content_types)

    def get_data_types(self, context):
        """Get selected data types"""
        data_types = set()
        if self.select_context_datatype:
            content = IWfTypedSharedContent(context, None)
            if content is not None:
                data_types.add(content.data_type)
        if self.selected_datatypes:
            data_types |= set(self.selected_datatypes)
        return list(data_types)

    def get_excluded_content_types(self, context):
        """Get excluded content types"""
        return list(self.excluded_content_types or ())

    def get_excluded_data_types(self, context):
        """Get excluded data types"""
        return list(self.excluded_datatypes or ())

    def get_results(self, context, sort_index=None, reverse=None, limit=None,
                    start=0, length=999, ignore_cache=False, get_count=False, request=None,
                    aggregates=None, settings=None, **kwargs):
        """Get query results"""
        count, aggregations, results = 0, {}, _MARKER
        if (not ignore_cache) and self.allow_user_params:
            if request is None:
                request = check_request()
            ignore_cache = bool(request.params)
        if not ignore_cache:
            # check for cache
            views_cache = get_cache(VIEWS_CACHE_NAME, VIEWS_CACHE_REGION, VIEWS_CACHE_NAMESPACE)
            if self.is_using_context:
                cache_key = VIEW_CONTEXT_CACHE_KEY.format(view=ICacheKeyValue(self),
                                                          context=ICacheKeyValue(context))
            else:
                cache_key = VIEW_CACHE_KEY.format(view=ICacheKeyValue(self))
            try:
                results = views_cache.get_value(cache_key)
                count = views_cache.get_value(f'{cache_key}::count')
                aggregations = views_cache.get_value(f'{cache_key}::aggregations')
            except KeyError:
                pass
        # Execute query
        if results is _MARKER:
            registry = get_pyramid_registry()
            adapter = registry.getAdapter(self, IViewQuery)
            if not sort_index:
                sort_index = self.order_by
            # Get query results
            count, aggregations, results = adapter.get_results(context,
                                                               sort_index,
                                                               reverse if reverse is not None
                                                                   else self.reversed_order,
                                                               limit or self.limit,
                                                               request=request,
                                                               aggregates=aggregates,
                                                               settings=settings,
                                                               **kwargs)
            count = min(count, limit or self.limit or 999)
            cache, results = tee(islice(results, start, start + length))
            if not ignore_cache:
                intids = get_utility(IIntIds)
                views_cache.set_value(cache_key, [intids.queryId(item) for item in cache])
                views_cache.set_value(f'{cache_key}::count', count)
                views_cache.set_value(f'{cache_key}::aggregations', aggregations)
                LOGGER.debug(f"Storing view items to cache key {cache_key}")
        else:
            results = CatalogResultSet(results)
            LOGGER.debug(f"Retrieving view items from cache key {cache_key}")
        return (count, aggregations, results) if get_count else results


@subscriber(IObjectModifiedEvent, context_selector=IWfView)
def handle_modified_view(event):
    """Invalidate views cache when a view is modified"""
    view = event.object
    views_cache = get_cache(VIEWS_CACHE_NAME, VIEWS_CACHE_REGION, VIEWS_CACHE_NAMESPACE)
    if view.is_using_context:
        views_cache.clear()
    else:
        intids = get_utility(IIntIds)
        cache_key = VIEW_CACHE_KEY.format(view=intids.queryId(view))
        views_cache.remove(cache_key)


@factory_config(IView)
@factory_config(ISharedContent, name=VIEW_CONTENT_TYPE)
class View(SharedContent):
    """Workflow managed view class"""

    content_type = VIEW_CONTENT_TYPE
    content_name = VIEW_CONTENT_NAME
    content_view = False



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.thesaurus.zmi.manager module

This module defines management components for thesaurus manager related forms.
"""

from zope.interface import alsoProvides

from pyams_content.component.thesaurus import ICollectionsManager, ICollectionsManagerTarget, \
    ITagsManager, ITagsManagerTarget, IThemesManager, IThemesManagerTarget
from pyams_content.interfaces import MANAGE_SITE_ROOT_PERMISSION
from pyams_content.zmi import content_js
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IFormContent, IGroup
from pyams_layer.interfaces import IPyAMSLayer
from pyams_utils.adapter import adapter_config
from pyams_utils.fanstatic import get_resource_path
from pyams_utils.interfaces.data import IObjectData
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminEditForm, FormGroupChecker
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IPropertiesMenu
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem

__docformat__ = 'restructuredtext'

from pyams_content import _


class BaseThesaurusManagerEditForm(AdminEditForm):
    """Base thesaurus manager edit form"""

    def update_widgets(self, prefix=None):
        # store thesaurus name in request header to be able to set
        # extract name correctly
        if self.request.method == 'POST':
            name = f'{self.prefix}widgets.thesaurus_name'
            value = self.request.params.get(name)
            if value is not None:
                self.request.headers['X-Thesaurus-Name'] = value
        super().update_widgets(prefix)
        name = self.widgets.get('thesaurus_name')
        if name is not None:
            name.object_data = {
                'ams-modules': {
                    'content': {
                        'src': get_resource_path(content_js)
                    }
                },
                'ams-change-handler': 'MyAMS.content.thesaurus.changeThesaurus'
            }
            alsoProvides(name, IObjectData)


#
# Tags management
#

@viewlet_config(name='tags-manager.menu',
                context=ITagsManagerTarget, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=730,
                permission=MANAGE_SITE_ROOT_PERMISSION)
class TagsManagerMenu(NavigationMenuItem):
    """Tags manager menu"""

    label = _("Tags settings")
    href = '#tags-manager.html'


@ajax_form_config(name='tags-manager.html',
                  context=ITagsManagerTarget, layer=IPyAMSLayer,
                  permission=MANAGE_SITE_ROOT_PERMISSION)
class TagsManagerEditForm(BaseThesaurusManagerEditForm):
    """Tags manager properties edit form"""

    title = _("Tags settings")
    legend = _("Selected tags thesaurus")

    fields = Fields(ITagsManager).select('thesaurus_name', 'extract_name')


@adapter_config(required=(ITagsManagerTarget, IAdminLayer, TagsManagerEditForm),
                provides=IFormContent)
def tags_manager_edit_form_content(context, request, form):
    """Tags manager edit form content getter"""
    return ITagsManager(context)


@adapter_config(name='glossary',
                required=(ITagsManagerTarget, IAdminLayer, TagsManagerEditForm),
                provides=IGroup)
class TagsManagerGlossaryGroup(FormGroupChecker):
    """Tags manager glossary group"""

    fields = Fields(ITagsManager).select('enable_glossary', 'glossary_thesaurus_name')
    checker_fieldname = 'enable_glossary'
    checker_mode = 'hide'


@adapter_config(required=(ITagsManagerTarget, IAdminLayer, TagsManagerGlossaryGroup),
                provides=IFormContent)
def tags_manager_glossary_group_content(context, request, group):
    """Tags manager glossary edit form group content getter"""
    return ITagsManager(context)


#
# Themes management
#

@viewlet_config(name='themes-manager.menu',
                context=IThemesManagerTarget, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=740,
                permission=MANAGE_SITE_ROOT_PERMISSION)
class ThemesManagerMenu(NavigationMenuItem):
    """Themes manager menu"""

    label = _("Themes settings")
    href = '#themes-manager.html'


@ajax_form_config(name='themes-manager.html',
                  context=IThemesManagerTarget, layer=IPyAMSLayer,
                  permission=MANAGE_SITE_ROOT_PERMISSION)
class ThemesManagerEditForm(BaseThesaurusManagerEditForm):
    """Themes manager properties edit form"""

    title = _("Themes settings")
    legend = _("Selected themes thesaurus")

    fields = Fields(IThemesManager)


@adapter_config(required=(IThemesManagerTarget, IAdminLayer, ThemesManagerEditForm),
                provides=IFormContent)
def themes_manager_edit_form_content(context, request, group):
    """Themes manager edit form content getter"""
    return IThemesManager(context)


#
# Collections management
#

@viewlet_config(name='collections-manager.menu',
                context=ICollectionsManagerTarget, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=750,
                permission=MANAGE_SITE_ROOT_PERMISSION)
class CollectionsManagerMenu(NavigationMenuItem):
    """Collections manager menu"""

    label = _("Collections settings")
    href = '#collections-manager.html'


@ajax_form_config(name='collections-manager.html',
                  context=ICollectionsManagerTarget, layer=IPyAMSLayer,
                  permission=MANAGE_SITE_ROOT_PERMISSION)
class CollectionsManagerEditForm(BaseThesaurusManagerEditForm):
    """Collections manager properties edit form"""

    title = _("Collections settings")
    legend = _("Selected collections thesaurus")

    fields = Fields(ICollectionsManager)


@adapter_config(required=(ICollectionsManagerTarget, IAdminLayer, CollectionsManagerEditForm),
                provides=IFormContent)
def collections_manager_edit_form_content(context, request, form):
    """Collections manager edit form content getter"""
    return ICollectionsManager(context)



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.seo module

This module provides SEO related classes and default adapters.
"""

from persistent import Persistent
from zope.container.contained import Contained
from zope.schema.fieldproperty import FieldProperty

from pyams_content.feature.seo.interfaces import ISEOContentInfo, SEO_INFO_ANNOTATION_KEY
from pyams_content.shared.common import IWfSharedContent
from pyams_utils.adapter import adapter_config, get_annotation_adapter
from pyams_utils.factory import factory_config

__docformat__ = 'restructuredtext'


@factory_config(ISEOContentInfo)
class SEOContentInfo(Persistent, Contained):
    """SEO content info"""

    include_sitemap = FieldProperty(ISEOContentInfo['include_sitemap'])


@adapter_config(required=IWfSharedContent,
                provides=ISEOContentInfo)
def shared_content_seo_info(context):
    """Shared content SEO info"""
    return get_annotation_adapter(context, SEO_INFO_ANNOTATION_KEY,
                                  ISEOContentInfo)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.video.interfaces module

This module defines interfaces which are required for support of external videos.
"""

from zope.annotation.interfaces import IAttributeAnnotatable
from zope.interface import Attribute, Interface
from zope.schema import TextLine, Choice

from pyams_content.component.paragraph import IBaseParagraph
from pyams_content.component.paragraph.interfaces import ParagraphRendererChoice
from pyams_i18n.schema import I18nTextField, I18nTextLineField

__docformat__ = 'restructuredtext'

from pyams_content import _


PYAMS_VIDEOS_PROVIDERS = 'pyams_content.video.providers'


class IExternalVideoSettings(Interface):
    """External video settings"""

    video_id = Attribute("Video ID")


class IExternalVideoProvider(Interface):
    """External video provider"""

    label = Attribute("Video provider label")
    weight = Attribute("Video provider weight (used for ordering)")
    settings_interface = Attribute("Video provider settings interface")


class IExternalVideo(IAttributeAnnotatable):
    """Base interface for external video integration"""

    author = TextLine(title=_("Author"),
                      description=_("Name of video's author"),
                      required=False)

    description = I18nTextField(title=_("Associated text"),
                                description=_("Video description displayed by front-office template"),
                                required=False)

    provider_name = Choice(title=_("Video provider"),
                           description=_("Name of external platform providing selected video"),
                           required=False,
                           vocabulary=PYAMS_VIDEOS_PROVIDERS)

    def get_provider(self):
        """Get external video provider utility"""

    settings = Attribute("Video settings")


EXTERNAL_VIDEO_PARAGRAPH_TYPE = 'video.external'
EXTERNAL_VIDEO_PARAGRAPH_NAME = _("External video")
EXTERNAL_VIDEO_PARAGRAPH_RENDERERS = 'PyAMS_content.paragraph.video.renderers'
EXTERNAL_VIDEO_PARAGRAPH_ICON_CLASS = 'fab fa-youtube'


class IExternalVideoParagraph(IExternalVideo, IBaseParagraph):
    """External video paragraph"""

    title = I18nTextLineField(title=_("Legend"),
                              required=False)

    renderer = ParagraphRendererChoice(description=_("Presentation template used for video"),
                                       renderers=EXTERNAL_VIDEO_PARAGRAPH_RENDERERS)



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.thesaurus.zmi module


"""

from zope.interface import Interface, alsoProvides, implementer

from pyams_content.component.thesaurus import ICollectionsInfo, ICollectionsManager, \
    ICollectionsTarget, ITagsInfo, ITagsManager, ITagsTarget, IThemesInfo, IThemesManager, \
    IThemesTarget
from pyams_content.component.thesaurus.zmi.interfaces import IThesaurusThemesEditForm
from pyams_content.shared.common.interfaces.types import ITypedSharedTool
from pyams_content.shared.common.zmi.types.interfaces import ISharedToolTypesTable
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IGroup, IInnerSubForm
from pyams_form.subform import InnerEditForm
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_table.interfaces import IColumn
from pyams_thesaurus.interfaces.thesaurus import IThesaurus
from pyams_thesaurus.zmi.widget import ThesaurusTermsTreeFieldWidget
from pyams_utils.adapter import adapter_config
from pyams_utils.registry import query_utility
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminEditForm, AdminModalEditForm, FormGroupChecker
from pyams_zmi.interfaces import IAdminLayer, TITLE_SPAN_BREAK
from pyams_zmi.interfaces.form import IFormTitle
from pyams_zmi.interfaces.viewlet import IPropertiesMenu
from pyams_zmi.table import ActionColumn
from pyams_zmi.utils import get_object_hint, get_object_label
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem


__docformat__ = 'restructuredtext'

from pyams_content import _


class BaseThesaurusTermsEditFormMixin:
    """Base thesaurus terms edit form"""

    label_css_class = 'hidden'
    input_css_class = 'col-12'

    manager = None
    interface = None
    fieldname = None

    @property
    def fields(self):
        """Fields getter"""
        fields = Fields(self.interface).select(self.fieldname)
        fields[self.fieldname].widget_factory = ThesaurusTermsTreeFieldWidget
        return fields

    def get_content(self):
        """Content getter"""
        return self.interface(self.context)

    def update_widgets(self, prefix=None):
        """Widgets update"""
        super().update_widgets(prefix)
        widget = self.widgets.get(self.fieldname)
        if widget is not None:
            manager = self.manager(self.request.root, None)
            if manager is not None:
                widget.thesaurus_name = manager.thesaurus_name
                widget.extract_name = manager.extract_name


#
# Tags management views
#

@viewlet_config(name='tags.menu',
                context=ITagsTarget, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=350,
                permission=VIEW_SYSTEM_PERMISSION)
class TagsMenu(NavigationMenuItem):
    """Tags menu"""

    def __new__(cls, context, request, view, manager):  # pylint: disable=unused-argument
        tags_manager = ITagsManager(request.root, None)
        if (tags_manager is None) or not tags_manager.thesaurus_name:
            return None
        return NavigationMenuItem.__new__(cls)

    label = _("Tags")
    href = '#tags.html'


class BaseTagsEditFormMixin(BaseThesaurusTermsEditFormMixin):
    """Base tags edit form"""

    legend = _("Content tags selection")

    manager = ITagsManager
    interface = ITagsInfo
    fieldname = 'tags'


@ajax_form_config(name='tags.html',
                  context=ITagsTarget, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class TagsEditForm(BaseTagsEditFormMixin, AdminEditForm):
    """Tags edit form"""

    title = _("Content tags")


@ajax_form_config(name='tags-modal.html',
                  context=ITagsTarget, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class TagsModalEditForm(BaseTagsEditFormMixin, AdminModalEditForm):
    """Tags modal edit form"""

    modal_class = 'modal-xl'
    subtitle = _("Content type default tags")


@adapter_config(required=(ITagsTarget, IAdminLayer, TagsModalEditForm),
                provides=IFormTitle)
def tags_edit_form_title(context, request, view):
    """Tags edit form title"""
    return TITLE_SPAN_BREAK.format(
        get_object_hint(context, request, view),
        get_object_label(context, request, view))


@adapter_config(name='tags',
                required=(ITagsInfo, IAdminLayer, BaseTagsEditFormMixin),
                provides=IThesaurus)
def tags_edit_form_thesaurus_adapter(context, request, view):  # pylint: disable=unused-argument
    """Tags edit form thesaurus adapter"""
    manager = ITagsManager(request.root, ITagsManager)
    if manager.thesaurus_name:
        return query_utility(IThesaurus, name=manager.thesaurus_name)
    return None


@adapter_config(name='tags',
                required=(ITypedSharedTool, IAdminLayer, ISharedToolTypesTable),
                provides=IColumn)
class SharedToolTypesTagsColumn(ActionColumn):
    """Shared tool data types table tags column"""

    hint = _("Default tags")
    icon_class = 'fas fa-tag'

    href = 'tags-modal.html'
    modal_target = True

    weight = 500


#
# Themes management views
#

@viewlet_config(name='themes.menu',
                context=IThemesTarget, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=360,
                permission=VIEW_SYSTEM_PERMISSION)
class ThemesMenu(NavigationMenuItem):
    """Themes menu"""

    def __new__(cls, context, request, view, manager):  # pylint: disable=unused-argument
        themes_manager = IThemesManager(request.root, None)
        if (themes_manager is None) or not themes_manager.thesaurus_name:
            return None
        return NavigationMenuItem.__new__(cls)

    label = _("Themes")
    href = '#themes.html'


class BaseThemesEditFormMixin(BaseThesaurusTermsEditFormMixin):
    """Base themes edit form mixin"""

    legend = _("Content themes selection")

    manager = IThemesManager
    interface = IThemesInfo
    fieldname = 'themes'

    def __init__(self, context, request):
        super().__init__(context, request)
        if not IThemesInfo(context).can_inherit:
            alsoProvides(self, IThesaurusThemesEditForm)
        else:
            self.legend = None


@ajax_form_config(name='themes.html',
                  context=IThemesTarget, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class ThemesEditForm(BaseThemesEditFormMixin, AdminEditForm):
    """Themes edit form"""

    title = _("Content themes")
    fields = Fields(Interface)


@ajax_form_config(name='themes-modal.html',
                  context=IThemesTarget, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class ThemesModalEditForm(BaseThemesEditFormMixin, AdminModalEditForm):
    """Themes modal edit form"""

    modal_class = 'modal-xl'
    subtitle = _("Content type default themes")
    fields = Fields(Interface)


@adapter_config(required=(IThemesTarget, IAdminLayer, ThemesModalEditForm),
                provides=IFormTitle)
def themes_edit_form_title(context, request, view):
    """Themes edit form title"""
    return TITLE_SPAN_BREAK.format(
        get_object_hint(context, request, view),
        get_object_label(context, request, view))


@adapter_config(name='themes-override',
                required=(IThemesTarget, IAdminLayer, BaseThemesEditFormMixin),
                provides=IGroup)
@implementer(IThesaurusThemesEditForm)
class ThemesEditFormInheritGroup(FormGroupChecker):
    """Themes edit form inherit group"""

    def __new__(cls, context, request, parent_form):
        if not IThemesInfo(context).can_inherit:
            return None
        return FormGroupChecker.__new__(cls)

    def __init__(self, context, request, parent_form):
        super().__init__(context, request, parent_form)
        self.legend = _("Don't inherit parent themes")

    fields = Fields(IThemesInfo).select('no_inherit')
    checker_fieldname = 'no_inherit'
    checker_mode = 'disable'


@adapter_config(name='themes-info',
                required=(IThemesTarget, IAdminLayer, IThesaurusThemesEditForm),
                provides=IInnerSubForm)
class ThemesInnerEditForm(BaseThemesEditFormMixin, InnerEditForm):
    """Themes inner edit form"""

    def __init__(self, context, request, parent_form):
        InnerEditForm.__init__(self, context, request, parent_form)
        if IThemesInfo(context).can_inherit:
            self.legend = None
            self.border_class = None


@adapter_config(name='themes',
                required=(IThemesInfo, IAdminLayer, BaseThemesEditFormMixin),
                provides=IThesaurus)
def themes_edit_form_thesaurus_adapter(context, request, view):  # pylint: disable=unused-argument
    """Themes edit form thesaurus adapter"""
    manager = IThemesManager(request.root, IThemesManager)
    if manager.thesaurus_name:
        return query_utility(IThesaurus, name=manager.thesaurus_name)
    return None


@adapter_config(name='themes',
                required=(ITypedSharedTool, IAdminLayer, ISharedToolTypesTable),
                provides=IColumn)
class SharedToolTypesThemesColumn(ActionColumn):
    """Shared tool data types table themes column"""

    hint = _("Default themes")
    icon_class = 'fas fa-tags'

    href = 'themes-modal.html'
    modal_target = True

    weight = 510


#
# Collections management views
#

@viewlet_config(name='collections.menu',
                context=ICollectionsTarget, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=370,
                permission=VIEW_SYSTEM_PERMISSION)
class CollectionsMenu(NavigationMenuItem):
    """Collections menu"""

    def __new__(cls, context, request, view, manager):  # pylint: disable=unused-argument
        collections_manager = ICollectionsManager(request.root, None)
        if (collections_manager is None) or not collections_manager.thesaurus_name:
            return None
        return NavigationMenuItem.__new__(cls)

    label = _("Collections")
    href = '#collections.html'


@ajax_form_config(name='collections.html',
                  context=ICollectionsTarget, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class CollectionsEditForm(BaseThesaurusTermsEditFormMixin, AdminEditForm):
    """Collections edit form"""

    legend = _("Content collections selection")

    manager = ICollectionsManager
    interface = ICollectionsInfo
    fieldname = 'collections'


@ajax_form_config(name='collections-modal.html',
                  context=ICollectionsTarget, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class CollectionsModalEditForm(BaseThemesEditFormMixin, AdminModalEditForm):
    """Collections modal edit form"""

    modal_class = 'modal-xl'
    subtitle = _("Content type default collections")


@adapter_config(required=(ICollectionsTarget, IAdminLayer, CollectionsModalEditForm),
                provides=IFormTitle)
def collections_edit_form_title(context, request, form):
    """Collections modal edit form title"""
    return TITLE_SPAN_BREAK.format(
        get_object_hint(context, request, form),
        get_object_label(context, request, form))


@adapter_config(name='collections',
                required=(ICollectionsInfo, IAdminLayer, CollectionsEditForm),
                provides=IThesaurus)
def collections_edit_form_thesaurus_adapter(context, request, view):  # pylint: disable=unused-argument
    """Collections edit form thesaurus adapter"""
    manager = ICollectionsManager(request.root, ICollectionsManager)
    if manager.thesaurus_name:
        return query_utility(IThesaurus, name=manager.thesaurus_name)
    return None



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.view.zmi module

This module defines components used for view properties management.
"""

import json

from zope.interface import Interface, alsoProvides

from pyams_content.shared.common.types import get_all_data_types
from pyams_content.shared.common.zmi import SharedContentPropertiesEditForm
from pyams_content.shared.view import IWfView
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IGroup
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_skin.interfaces.viewlet import IHelpViewletManager
from pyams_skin.viewlet.help import AlertMessage
from pyams_utils.adapter import adapter_config
from pyams_utils.interfaces.data import IObjectData
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import FormGroupSwitcher
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IContentManagementMenu, IMenuHeader

__docformat__ = 'restructuredtext'

from pyams_content import _


@adapter_config(required=(IWfView, IAdminLayer, Interface, IContentManagementMenu),
                provides=IMenuHeader)
def view_management_menu_header(context, request, view, manager):
    """View management menu header"""
    return request.localizer.translate(_("View management"))


@ajax_form_config(name='properties.html',
                  context=IWfView, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class ViewPropertiesEditForm(SharedContentPropertiesEditForm):
    """View properties edit form"""

    interface = IWfView


@adapter_config(name='view-properties',
                required=(IWfView, IAdminLayer, ViewPropertiesEditForm),
                provides=IGroup)
class ViewPropertiesGroup(FormGroupSwitcher):
    """View properties group"""

    legend = _("View settings")

    fields = Fields(IWfView).select('select_context_path', 'select_context_type',
                                    'selected_content_types', 'select_context_datatype',
                                    'selected_datatypes', 'excluded_content_types',
                                    'excluded_datatypes', 'allow_user_params',
                                    'order_by', 'reversed_order', 'limit', 'age_limit')
    switcher_mode = 'always'

    def update_widgets(self, prefix=None, use_form_mode=True):
        super().update_widgets(prefix, use_form_mode)
        for field in ('selected_datatypes', 'excluded_datatypes'):
            all_datatypes = get_all_data_types(self.request, self.context, field)
            datatypes = self.widgets.get(field)
            if datatypes is not None:
                datatypes.object_data = {
                    'data': json.dumps(all_datatypes)
                }
                alsoProvides(datatypes, IObjectData)


@viewlet_config(name='view-properties.help',
                context=IWfView, layer=IAdminLayer, view=ViewPropertiesGroup,
                manager=IHelpViewletManager, weight=10)
class ViewPropertiesHelp(AlertMessage):
    """View properties help"""

    status = 'info'
    _message = _("These settings apply to search made by the view.\n"
                 "If you select the option \"Include ONLY selected references\", "
                 "via the \"References\" menu, only selected references will be selected "
                 "(if published) and no real search will be made !\n"
                 "In other modes, and if no search criteria is defined (in this form, or "
                 "in other settings forms), ALL published contents will be selected by the view !!")

    message_renderer = 'text'



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.site.zmi.topic module

This module defines sites topics management components.
"""

from zope.intid.interfaces import IIntIds
from zope.schema import Int

from pyams_content.interfaces import CREATE_CONTENT_PERMISSION
from pyams_content.shared.common.zmi import SharedContentAddForm
from pyams_content.shared.site.interfaces import ISiteContainer, ISiteManager, ISiteTopic, IWfSiteTopic
from pyams_content.shared.site.zmi.interfaces import ISiteTreeTable
from pyams_content.shared.site.zmi.widget.folder import SiteManagerFoldersSelectorFieldWidget
from pyams_content.zmi.interfaces import IDashboardColumn, IDashboardContentVisibility
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_layer.interfaces import IPyAMSLayer
from pyams_skin.viewlet.menu import MenuItem
from pyams_utils.adapter import NullAdapter, adapter_config
from pyams_utils.registry import get_utility
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyams_workflow.interfaces import IWorkflowVersions
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.table import ITableElementEditor
from pyams_zmi.interfaces.viewlet import IContextAddingsViewletManager, IToolbarViewletManager
from pyams_zmi.table import TableElementEditor

__docformat__ = 'restructuredtext'

from pyams_content import _


@viewlet_config(name='add-content.action',
                context=ISiteManager, layer=IAdminLayer,
                manager=IToolbarViewletManager, weight=10)
class SiteManagerContentAddAction(NullAdapter):
    """Site manager add action"""


class ISiteTopicAddFormFields(IWfSiteTopic):
    """Site topic add form fields"""

    parent = Int(title=_("Parent"),
                 description=_("Topic's parent"),
                 required=True)


@adapter_config(required=IWfSiteTopic,
                provides=ISiteTopicAddFormFields)
def site_topic_add_form_fields_adapter(context):
    """Site topic add form fields adapter"""
    return context


@viewlet_config(name='add-site-topic.action',
                context=ISiteContainer, layer=IAdminLayer, view=ISiteTreeTable,
                manager=IContextAddingsViewletManager, weight=20,
                permission=CREATE_CONTENT_PERMISSION)
class SiteContainerTopicAddMenuItem(MenuItem):
    """Site container topic add menu item"""

    label = _("Add site topic...")
    icon_class = 'far fa-file'

    href = 'add-site-topic.html'
    modal_target = True


@ajax_form_config(name='add-site-topic.html',
                  context=ISiteContainer, layer=IPyAMSLayer,
                  permission=CREATE_CONTENT_PERMISSION)
class SiteContainerTopicAddForm(SharedContentAddForm):
    """Site container topic add form"""

    fields = Fields(ISiteTopicAddFormFields).select('title', 'data_type', 'parent', 'notepad')
    fields['parent'].widget_factory = SiteManagerFoldersSelectorFieldWidget

    @property
    def container_target(self):
        """new content container getter"""
        return self.__parent if self.__parent is not None else self.context

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        if 'parent' in self.widgets:
            self.widgets['parent'].permission = CREATE_CONTENT_PERMISSION

    def update_content(self, obj, data):
        data = data.get(self, data)
        # initialize new topic
        intids = get_utility(IIntIds)
        self.__parent = intids.queryObject(data.pop('parent'))
        super().update_content(obj, data)

    def next_url(self):
        """Redirect URL getter"""
        return absolute_url(self.__parent, self.request,
                            f'{self.__uuid}/++versions++/1/admin')


@adapter_config(required=(ISiteTopic, IAdminLayer, ISiteTreeTable),
                provides=ITableElementEditor)
class SiteTopicTableElementEditor(TableElementEditor):
    """Site topic table element editor"""

    view_name = 'admin'
    modal_target = False

    @property
    def href(self):
        """Target URL getter"""
        versions = IWorkflowVersions(self.context)
        version = versions.get_last_versions(1)[0]
        return absolute_url(version, self.request, self.view_name)


@adapter_config(required=(ISiteTopic, IAdminLayer, IDashboardColumn),
                provides=IDashboardContentVisibility)
def site_topic_dashboard_visibility(context, request, column):
    """Site topic dashboard visibility"""
    return column.has_permission(context), ''



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from pyams_content.feature.filter import IFilterAggregate
from pyams_content.feature.filter.interfaces import IAggregatedPortletRendererSettings, IFiltersContainer
from pyams_content.shared.view.portlet import IViewItemsAggregates
from pyams_utils.adapter import adapter_config


@adapter_config(required=IAggregatedPortletRendererSettings,
                provides=IViewItemsAggregates)
def aggregated_portlet_settings_aggregates(settings):
    """Aggregated portlet settings aggregates"""
    container = IFiltersContainer(settings, None)
    if container is None:
        return None
    for visible_filter in container.get_visible_filters():
        aggregate = IFilterAggregate(visible_filter, None)
        if aggregate is not None:
            yield aggregate



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.paragraph.html.skin.interfaces module

This module defines interfaces of HTML paragraph default renderer.
"""

from zope.schema import Choice

from pyams_content.feature.renderer import IRendererSettings


__docformat__ = 'restructuredtext'

from pyams_content import _


class IHTMLParagraphRendererSettings(IRendererSettings):
    """HTML paragraph renderer settings"""

    xs_horizontal_padding = Choice(title=_("XS horizontal padding"),
                                   description=_("Define horizontal padding for smartphones"),
                                   required=False,
                                   values=list(range(5)))

    sm_horizontal_padding = Choice(title=_("SM horizontal padding"),
                                   description=_("Define horizontal padding for tablets"),
                                   required=False,
                                   values=list(range(5)))

    md_horizontal_padding = Choice(title=_("MD horizontal padding"),
                                   description=_("Define horizontal padding for medium screens"),
                                   required=False,
                                   values=list(range(5)))

    lg_horizontal_padding = Choice(title=_("LG horizontal padding"),
                                   description=_("Define horizontal padding for large screens"),
                                   required=False,
                                   values=list(range(5)))

    xl_horizontal_padding = Choice(title=_("XL horizontal padding"),
                                   description=_("Define horizontal padding for extra large "
                                                 "screens"),
                                   required=False,
                                   values=list(range(5)))

    def has_padding(self):
        """Check if any padding is defined"""

    def get_padding(self):
        """Get settings padding"""



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.site.zmi.tree module

This module defines components which are used to display the whole site tree.
"""

import json

from pyramid.decorator import reify
from pyramid.interfaces import IView
from pyramid.location import lineage
from pyramid.view import view_config
from zope.interface import implementer
from zope.intid import IIntIds
from zope.lifecycleevent import ObjectMovedEvent

from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION, MANAGE_SITE_PERMISSION
from pyams_content.shared.site.interfaces import IBaseSiteItem, ISiteContainer, ISiteElement, \
    ISiteManager, SITE_CONTAINER_REDIRECT_MODE
from pyams_content.shared.site.zmi.interfaces import ISiteTreeTable
from pyams_content.zmi import content_js
from pyams_content.zmi.dashboard import DashboardContentNumberColumn, DashboardContentOwnerColumn, \
    DashboardContentStatusColumn, DashboardContentStatusDatetimeColumn, \
    DashboardContentStatusPrincipalColumn, DashboardContentTimestampColumn, \
    DashboardContentTypeColumn, DashboardContentVersionColumn, DashboardLabelColumn, \
    DashboardVisibilityColumn
from pyams_content.zmi.interfaces import IDashboardColumn, IDashboardContentType, \
    IDashboardContentVisibility, IDashboardTable, IDashboardView
from pyams_layer.interfaces import IPyAMSLayer
from pyams_layer.skin import apply_skin
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_site.interfaces import ISiteRoot
from pyams_table.interfaces import IColumn, IValues
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.fanstatic import get_resource_path
from pyams_utils.interfaces.traversing import IPathElements
from pyams_utils.registry import get_utility
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyams_workflow.interfaces import IWorkflowPublicationInfo
from pyams_zmi.helper.container import delete_container_element, switch_element_attribute
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.table import IReorderColumn
from pyams_zmi.interfaces.viewlet import ISiteManagementMenu
from pyams_zmi.skin import AdminSkin
from pyams_zmi.table import ReorderColumn, Table, TableAdminView, TrashColumn, get_table_id
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem

__docformat__ = 'restructuredtext'

from pyams_content import _


def get_item_order(item):
    """Get item order value"""
    for item in reversed(list(lineage(item))):
        parent = get_parent(item, ISiteContainer)
        if parent is None:
            continue
        index = list(parent.__parent__.keys()).index(parent.__name__)
        yield f'{index:03}'


@implementer(IDashboardTable, ISiteTreeTable, IView)
class SiteContainerTreeTable(Table):
    """Site container tree table"""

    @property
    def id(self):
        """Table id getter"""
        manager = get_parent(self.context, ISiteManager)
        return get_table_id(self, manager)

    permission = MANAGE_SITE_PERMISSION
    display_if_empty = True

    can_sort = False

    @property
    def css_classes(self):
        classes = super().css_classes.copy()
        classes.update({
            'tr.selected': lambda item, col, row: 'bg-secondary' if item is self.context else ''
        })
        return classes

    @reify
    def data_attributes(self):
        attributes = super().data_attributes
        intids = get_utility(IIntIds)
        manager = get_parent(self.context, ISiteManager)
        attributes.setdefault('table', {}).update({
            'data-ams-modules': json.dumps({
                'tree': 'tree',
                'content': {
                    'src': get_resource_path(content_js)
                }
            }),
            'data-ams-location': absolute_url(self.context, self.request),
            'data-searching': 'false',
            'data-length-change': 'false',
            'data-info': 'false',
            'data-paging': 'false',
            'data-ams-order': '0,asc',
            'data-ams-visible': json.dumps(IWorkflowPublicationInfo(self.context).is_published()),
            'data-ams-tree-node-id': intids.queryId(manager)
        })
        attributes.setdefault('tr', {}).update({
            'data-ams-location': lambda x, col: absolute_url(x.__parent__, self.request),
            'data-ams-tree-node-id': lambda x, col: intids.queryId(x),
            'data-ams-tree-node-parent-id': lambda x, col: intids.queryId(x.__parent__)
        })
        attributes.setdefault('td', {}).update({
            'data-order': lambda x, col: ':'.join(get_item_order(x))
                if IReorderColumn.providedBy(col) else None
        })
        permission = self.permission
        if self.can_sort and \
                ((not permission) or
                 self.request.has_permission(permission, self.context)):
            attributes['table'].update({
                'data-row-reorder': json.dumps({
                    'update': False
                }),
                'data-ams-reordered': 'MyAMS.tree.sortTree',
                'data-ams-reorder-url': 'set-site-order.json'
            })
            attributes['tr'].update({
                'data-ams-row-value': lambda x, col: x.__name__
            })
        return attributes


@adapter_config(required=(IBaseSiteItem, IAdminLayer, SiteContainerTreeTable),
                provides=IValues)
class SiteContainerTreeTableValues(ContextRequestViewAdapter):
    """Site container tree table values"""

    @property
    def values(self):
        """Site container tree table values getter"""

        def get_values(container, result):
            if container not in result:
                result.append(container)
            if ISiteContainer.providedBy(container) and (container in lineage(self.context)):
                for child in container.values():
                    get_values(child, result)
            return result

        manager = get_parent(self.context, ISiteManager)
        values = []
        for item in manager.values():
            values.append(item)
            if ISiteContainer.providedBy(item):
                get_values(item, values)
        yield from values


@adapter_config(name='reorder',
                required=(IBaseSiteItem, IAdminLayer, SiteContainerTreeTable),
                provides=IColumn)
class SiteContainerTreeReorderColumn(ReorderColumn):
    """Site container tree reorder column"""

    permission = MANAGE_SITE_PERMISSION

    def render_cell(self, item):
        if not self.table.can_sort:
            return ''
        return super().render_cell(item)


@adapter_config(name='visible',
                required=(IBaseSiteItem, IAdminLayer, SiteContainerTreeTable),
                provides=IColumn)
class SiteContainerTreeVisibleColumn(DashboardVisibilityColumn):
    """Site container tree visible column"""


@adapter_config(required=(IBaseSiteItem, IAdminLayer, IDashboardColumn),
                provides=IDashboardContentVisibility)
def site_item_dashboard_visibility(context, request, column):
    """Site item dashboard visibility"""
    info = IWorkflowPublicationInfo(context, None)
    if info is None:
        return False, ''
    if info.is_published():
        icon_class = f"{column.object_data.get('ams-icon-on')}"
    else:
        icon_class = f"{column.object_data.get('ams-icon-off')} text-danger opaque"
    hint = column.get_icon_hint(context)
    return False, f'<i class="fa-fw {icon_class} hint align-base" title="{hint}"></i>'


@adapter_config(name='name',
                required=(IBaseSiteItem, IAdminLayer, SiteContainerTreeTable),
                provides=IColumn)
class SiteContainerTreeNameColumn(DashboardLabelColumn):
    """Site container tree name column"""

    i18n_header = _("Folders contents")
    sortable = 'false'

    css_classes = {
        'td': 'text-truncate'
    }

    def render_head_cell(self):
        return '''<span data-ams-stop-propagation="true"
            data-ams-click-handler="MyAMS.tree.switchTree">
            <span class="small hint" title="{hint}" data-ams-hint-gravity="e">
                <span class="switcher">
                    <i class="far fa-plus-square"></i>
                </span>
            </span>&nbsp;&nbsp;{title}
        </span>'''.format(
            hint=self.request.localizer.translate(_("Click to open/close all folders")),
            title=super().render_head_cell())

    def render_cell(self, item, name=None):
        depth = -3
        for _node in lineage(item):
            depth += 1
        intids = get_utility(IIntIds)
        item_id = intids.queryId(item)
        new_parent = int(self.request.params.get('parent', 0))
        if new_parent == 0:
            parents = map(lambda x: intids.queryId(x), lineage(self.context))
        else:
            parents = map(lambda x: intids.queryId(x), lineage(intids.queryObject(new_parent)))
        translate = self.request.localizer.translate
        expanded = (item is self.context) or (item_id == new_parent) or (item_id in parents)
        return '''<div class="name">
            {padding}
            <span class="small hint tree-switcher" title="{hint}" data-ams-hint-gravity="e" 
                  data-ams-click-handler="MyAMS.tree.switchTreeNode"
                  data-ams-stop-propagation="true">{switch}</span>
            &nbsp;&nbsp;<span class="title">{title}</span> {arrow}
        </div>'''.format(
            padding='<span class="tree-node-padding"></span>' * depth,
            hint=translate(_("Click to show/hide inner folders")),
            switch='<span class="switcher {switch_state}">'
                   '<i class="far fa-{state}-square switch"></i>'
                   '</span>'.format(
                switch_state='expanded' if expanded else '',
                state=getattr(item, '_v_state',
                              'minus' if expanded else 'plus'))
                    if ISiteContainer.providedBy(item) else '',
            title=name or super().render_cell(item),
            arrow='<i class="ml-1 fas fa-share fa-rotate-90 text-muted hint" '
                  '   data-original-title="{title}"></i>'.format(
                title=translate(_("This container automatically redirects navigation to "
                                  "it's first visible content"))) if
                    ISiteContainer.providedBy(item) and
                    (item.navigation_mode == SITE_CONTAINER_REDIRECT_MODE) else '')


@adapter_config(name='content-type',
                required=(IBaseSiteItem, IAdminLayer, ISiteTreeTable),
                provides=IColumn)
class SiteContainerContentTypeColumn(DashboardContentTypeColumn):
    """Site container content type column"""

    sortable = 'false'


@adapter_config(required=(ISiteElement, IAdminLayer, IDashboardColumn),
                provides=IDashboardContentType)
def site_item_content_type(context, request, column):
    """Site item content type getter"""
    return request.localizer.translate(context.content_name)


@adapter_config(name='sequence',
                required=(IBaseSiteItem, IAdminLayer, ISiteTreeTable),
                provides=IColumn)
class SiteContainerContentNumberColumn(DashboardContentNumberColumn):
    """Site container content number column"""

    sortable = 'false'


@adapter_config(name='status',
                required=(IBaseSiteItem, IAdminLayer, ISiteTreeTable),
                provides=IColumn)
class SiteContainerContentStatusColumn(DashboardContentStatusColumn):
    """Site container content status column"""

    sortable = 'false'


@adapter_config(name='status-datetime',
                required=(IBaseSiteItem, IAdminLayer, ISiteTreeTable),
                provides=IColumn)
class SiteContainerStatusDatetimeColumn(DashboardContentStatusDatetimeColumn):
    """Site container content status datetime column"""

    sortable = 'false'


@adapter_config(name='version',
                required=(IBaseSiteItem, IAdminLayer, ISiteTreeTable),
                provides=IColumn)
class SiteContainerVersionColumn(DashboardContentVersionColumn):
    """Site container content version column"""

    sortable = 'false'


@adapter_config(name='status-principal',
                required=(IBaseSiteItem, IAdminLayer, ISiteTreeTable),
                provides=IColumn)
class SiteContainerStatusPrincipalColumn(DashboardContentStatusPrincipalColumn):
    """Site container content status principal column"""

    sortable = 'false'


@adapter_config(name='owner',
                required=(IBaseSiteItem, IAdminLayer, ISiteTreeTable),
                provides=IColumn)
class SiteContainerOwnerColumn(DashboardContentOwnerColumn):
    """Site container content owner column"""

    sortable = 'false'


@adapter_config(name='timestamp',
                required=(IBaseSiteItem, IAdminLayer, ISiteTreeTable),
                provides=IColumn)
class SiteContainerTimestampColumn(DashboardContentTimestampColumn):
    """Site container content timestamp column"""

    sortable = 'false'


@adapter_config(name='trash',
                required=(IBaseSiteItem, IPyAMSLayer, ISiteTreeTable),
                provides=IColumn)
class SiteContainerTreeTrashColumn(TrashColumn):
    """Site container tree trash column"""

    hint = _("Delete site item")
    permission = MANAGE_SITE_PERMISSION

    def has_permission(self, item):
        if (item is self.context) or (item in lineage(self.context)):
            return False
        return super().has_permission(item) and item.is_deletable()


@pagelet_config(name='site-tree.html',
                context=IBaseSiteItem, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
@implementer(IDashboardView)
class SiteManagerTreeView(TableAdminView):
    """Site manager tree view"""

    title = _("Site tree")

    table_class = SiteContainerTreeTable
    table_label = _("Site content")

    def __init__(self, context, request, *args, **kwargs):
        super().__init__(context, request, *args, **kwargs)
        self.table.can_sort = ISiteManager.providedBy(context)


@viewlet_config(name='site-tree.menu',
                context=IBaseSiteItem, layer=IAdminLayer,
                manager=ISiteManagementMenu, weight=5,
                permission=VIEW_SYSTEM_PERMISSION)
class SiteManagerTreeMenu(NavigationMenuItem):
    """Site manager tree menu"""

    label = _("Site tree")
    icon_class = 'fas fa-sitemap'
    href = '#site-tree.html'


@view_config(name='get-tree-nodes.json',
             context=ISiteContainer, request_type=IPyAMSLayer,
             permission=VIEW_SYSTEM_PERMISSION, renderer='json', xhr=True)
def get_tree_nodes(request):
    """Get tree nodes"""
    apply_skin(request, AdminSkin)
    table = SiteContainerTreeTable(request.context, request)
    table.can_sort = json.loads(request.params.get('can_sort', 'false'))
    table.update()
    result = []
    for item in request.context.values():
        row = table.setup_row(item)
        result.append(table.render_row(row).strip())
    return result


@view_config(name='get-tree.json',
             context=IBaseSiteItem, request_type=IPyAMSLayer,
             permission=VIEW_SYSTEM_PERMISSION, renderer='json', xhr=True)
def get_tree(request):
    """Get whole tree nodes"""

    def get_tree_values(parent):
        """Iterator over container tree items"""
        for item in parent.values():
            setattr(item, '_v_state', 'minus')
            yield item
            delattr(item, '_v_state')
            if ISiteContainer.providedBy(item):
                yield from get_tree_values(item)

    apply_skin(request, AdminSkin)
    table = SiteContainerTreeTable(request.context, request)
    table.can_sort = json.loads(request.params.get('can_sort', 'false'))
    table.update()
    result = []
    manager = get_parent(request.context, ISiteManager)
    for item in get_tree_values(manager):
        row = table.setup_row(item)
        result.append(table.render_row(row).strip())
    return result


@view_config(name='set-site-order.json',
             context=IBaseSiteItem, request_type=IPyAMSLayer,
             permission=MANAGE_SITE_PERMISSION, renderer='json', xhr=True)
def set_site_order(request):
    """Set site elements order"""
    apply_skin(request, AdminSkin)
    intids = get_utility(IIntIds)
    action = request.params.get('action')
    parent_oid = int(request.params.get('parent'))
    old_parent = None
    new_parent = intids.queryObject(parent_oid)
    # check for changing parent
    if action == 'reparent':
        child_oid = int(request.params.get('child'))
        child = intids.queryObject(child_oid)
        # check if new parent is not a previous child
        parent_path = IPathElements(new_parent)
        if child_oid in parent_path.parents:
            return {
                'status': 'reload',
                'smallbox': {
                    'status': 'error',
                    'message': request.localizer.translate(_("Can't reparent object to one of "
                                                             "it's children. Reloading...")),
                    'timeout': 5000
                }
            }
        old_parent = child.__parent__
        new_name = old_name = child.__name__
        if old_name in new_parent:
            index = 1
            new_name = '{name}-{index:02}'.format(name=old_name, index=index)
            while new_name in new_parent:
                index += 1
                new_name = '{name}-{index:02}'.format(name=old_name, index=index)
        new_parent[new_name] = child
        del old_parent[old_name]
        request.registry.notify(ObjectMovedEvent(child, old_parent, old_name,
                                                 new_parent, new_name))
    # Re-define order
    if len(new_parent.keys()) > 1:
        names = [
            child.__name__
            for child in [
                intids.queryObject(oid)
                for oid in map(int, json.loads(request.params.get('order')))
            ]
            if (child is not None) and (child.__parent__ is new_parent)
        ]
        if names:
            new_parent.updateOrder(names)
    # get old and new parents children
    table = SiteContainerTreeTable(request.context, request)
    table.can_sort = json.loads(request.params.get('can_sort', 'false'))
    table.update()
    result = []
    items = set()
    parents = [new_parent]
    if action == 'reparent':
        parents.append(old_parent)
    for parent in parents:
        for item in reversed(list(lineage(parent))):
            if ISiteRoot.providedBy(item) or (item in items):
                continue
            if not ISiteManager.providedBy(item):
                setattr(item, '_v_state', 'minus')
                row = table.setup_row(item)
                result.append(table.render_row(row).strip())
                items.add(item)
                delattr(item, '_v_state')
            for child in item.values():
                row = table.setup_row(child)
                result.append(table.render_row(row).strip())
    return result


@view_config(name='switch-visible-item.json',
             context=ISiteContainer, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION,
             renderer='json', xhr=True)
def switch_visible_item(request):
    """Switch visible item"""
    result = switch_element_attribute(request)
    if result['status'] == 'success':
        result.pop('status')
        result.pop('message')
        intids = get_utility(IIntIds)
        name = request.params.get('object_name')
        container = request.context
        node_id = intids.queryId(container[name])
        result.setdefault('callbacks', []).append({
            'module': 'content',
            'callback': 'MyAMS.content.tree.switchVisibleElement',
            'options': {
                'node_id': node_id,
                'state': result.pop('state')
            }
        })
    return result


@view_config(name='delete-element.json',
             context=ISiteContainer, request_type=IPyAMSLayer,
             permission=MANAGE_SITE_PERMISSION, renderer='json', xhr=True)
def delete_site_item(request):
    """Delete item from site container"""
    intids = get_utility(IIntIds)
    container = request.context
    name = request.params.get('object_name')
    node_id = intids.queryId(container[name]) if name in container else None
    result = delete_container_element(request, ignore_permission=True)
    if result.get('status') == 'success':
        result.setdefault('callbacks', []).append({
            'module': 'tree',
            'callback': 'MyAMS.tree.deleteElement',
            'options': {
                'node_id': node_id
            }
        })
        result['handle_json'] = True
    return result



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.view.query module

This module defines adapters which are used to handle catalog-based
views queries. These adapters can be overriden, for example by packages
providing Elasticsearch support (see :py:mod:`pyams_content_es`).
"""

from datetime import datetime, timedelta, timezone

from hypatia.catalog import CatalogQuery
from hypatia.interfaces import ICatalog
from hypatia.query import Any, Eq, Gt, Lt, NotAny, Or

from pyams_catalog.query import CatalogResultSet, or_
from pyams_content.shared.view import IViewQuery, IWfView
from pyams_content.shared.view.interfaces import RELEVANCE_ORDER
from pyams_content.shared.view.interfaces.query import END_PARAMS_MARKER, IViewQueryFilterExtension, \
    IViewQueryParamsExtension, IViewUserQuery
from pyams_utils.adapter import ContextAdapter, adapter_config, get_adapter_weight
from pyams_utils.list import unique_iter
from pyams_utils.registry import get_pyramid_registry, get_utility
from pyams_utils.timezone import tztime
from pyams_workflow.interfaces import IWorkflow

__docformat__ = 'restructuredtext'


@adapter_config(required=IWfView,
                provides=IViewQuery)
class ViewQuery(ContextAdapter):
    """View query"""

    def get_params(self, context, request=None, **kwargs):
        view = self.context
        catalog = get_utility(ICatalog)
        registry = get_pyramid_registry()
        # check publication dates
        now = tztime(datetime.now(timezone.utc))
        params = Lt(catalog['effective_date'], now)
        # check workflow states
        if 'state' in kwargs:
            state = kwargs['state']
            if not isinstance(state, (list, tuple, set)):
                state = (state,)
            params &= Any(catalog['workflow_state'], state)
        else:
            wf_params = None
            for workflow in registry.getAllUtilitiesRegisteredFor(IWorkflow):
                wf_params = or_(wf_params, Any(catalog['workflow_state'], workflow.visible_states))
            params &= wf_params
        # check custom extensions
        get_all_params = True
        for name, adapter in sorted(registry.getAdapters((view,), IViewQueryParamsExtension),
                                    key=get_adapter_weight):
            for new_params in adapter.get_params(context, request):
                if new_params is None:
                    return None
                elif new_params is END_PARAMS_MARKER:
                    get_all_params = False
                    break
                else:
                    params &= new_params
            else:
                continue
            break
        # activate search
        filters = Or(Eq(catalog['push_end_date'], None),
                     Gt(catalog['push_end_date'], now))
        if get_all_params:
            # check content path
            content_path = view.get_content_path(context)
            if content_path is not None:
                filters &= Eq(catalog['parents'], content_path)
            # check content types
            if 'content_type' in kwargs:
                filters &= Eq(catalog['content_type'], kwargs['content_type'])
            else:
                filters &= NotAny(catalog['content_type'], set(view.get_ignored_types()))
                content_types = view.get_content_types(context)
                if content_types:
                    filters &= Any(catalog['content_type'], content_types)
            # check data types
            data_types = view.get_data_types(context)
            if data_types:
                filters &= Any(catalog['data_type'], data_types)
            # check excluded content types
            content_types = view.get_excluded_content_types(context)
            if content_types:
                filters &= NotAny(catalog['content_type'], content_types)
            # check excluded data types
            data_types = view.get_excluded_data_types(context)
            if data_types:
                filters &= NotAny(catalog['data_type'], data_types)
            # check age limit
            age_limit = view.age_limit
            if age_limit:
                filters &= Gt(catalog['content_publication_date'],
                              now - timedelta(days=age_limit))
        params &= filters
        return params

    def get_results(self, context, sort_index, reverse, limit,
                    request=None, aggregates=None, settings=None, **kwargs):
        view = self.context
        catalog = get_utility(ICatalog)
        registry = get_pyramid_registry()
        params = self.get_params(context, request, **kwargs)
        if params is None:
            items = CatalogResultSet([])
            total_count = 0
        else:
            if (not sort_index) or (sort_index == RELEVANCE_ORDER):
                sort_index = None
            query = CatalogQuery(catalog).query(params,
                                                sort_index=sort_index,
                                                reverse=reverse,
                                                limit=limit)
            total_count = query[0]
            items = CatalogResultSet(query)
        for name, adapter in sorted(registry.getAdapters((view,), IViewQueryFilterExtension),
                                    key=get_adapter_weight):
            items = adapter.filter(context, items, request)
        return unique_iter(items), total_count, {}


@adapter_config(name='user-params',
                required=IWfView,
                provides=IViewQueryParamsExtension)
class UserViewQueryParamsExtension(ContextAdapter):
    """User view query params extension"""

    weight = 999

    def __new__(cls, context):
        if not context.allow_user_params:
            return None
        return ContextAdapter.__new__(cls)

    def get_params(self, context, request=None):
        """User params getter"""
        registry = get_pyramid_registry()
        for name, adapter in sorted(registry.getAdapters((self.context,), IViewUserQuery),
                                    key=get_adapter_weight):
            yield from adapter.get_user_params(request)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.video.paragraph module

This module provides components which are used to handle integration of external
videos as content paragraphs.
"""

from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.paragraph import BaseParagraph, IBaseParagraph
from pyams_content.component.video import ExternalVideo
from pyams_content.component.video.interfaces import EXTERNAL_VIDEO_PARAGRAPH_ICON_CLASS, \
    EXTERNAL_VIDEO_PARAGRAPH_NAME, EXTERNAL_VIDEO_PARAGRAPH_RENDERERS, EXTERNAL_VIDEO_PARAGRAPH_TYPE, \
    IExternalVideoParagraph
from pyams_content.feature.renderer import RenderersVocabulary
from pyams_utils.factory import factory_config
from pyams_utils.vocabulary import vocabulary_config

__docformat__ = 'restructuredtext'


@factory_config(IExternalVideoParagraph)
@factory_config(IBaseParagraph, name=EXTERNAL_VIDEO_PARAGRAPH_TYPE)
class ExternalVideoParagraph(ExternalVideo, BaseParagraph):
    """External video paragraph"""

    factory_name = EXTERNAL_VIDEO_PARAGRAPH_TYPE
    factory_label = EXTERNAL_VIDEO_PARAGRAPH_NAME
    factory_intf = IExternalVideoParagraph

    icon_class = EXTERNAL_VIDEO_PARAGRAPH_ICON_CLASS

    renderer = FieldProperty(IExternalVideoParagraph['renderer'])


@vocabulary_config(name=EXTERNAL_VIDEO_PARAGRAPH_RENDERERS)
class ExternalVideoParagraphRenderersVocabulary(RenderersVocabulary):
    """External video paragraph renderers vocabulary"""

    content_interface = IExternalVideoParagraph



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.file module

This module provides a custom file view to check parents publication.
"""

__docformat__ = 'restructuredtext'

from pyramid.httpexceptions import HTTPNotFound
from pyramid.location import lineage
from pyramid.view import view_config

from pyams_file.interfaces import IFile
from pyams_file.skin.view import FileView
from pyams_layer.interfaces import IPyAMSUserLayer
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_workflow.interfaces import IWorkflowPublicationInfo


@view_config(context=IFile,
             request_type=IPyAMSUserLayer)
def ProtectedFileView(request):  # pylint: disable=invalid-name
    """Protected file view"""
    context = request.context
    if not request.has_permission(VIEW_SYSTEM_PERMISSION, context=context):  # authenticated
        for parent in lineage(context):
            publication_info = IWorkflowPublicationInfo(parent, None)
            if (publication_info is not None) and not publication_info.is_visible(request):
                raise HTTPNotFound()
    return FileView(request)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.view.zmi.thesaurus module

This module defines management interface components used to handle
thesaurus-based views settings.
"""

from pyams_content.component.thesaurus import ICollectionsManager, ITagsManager, IThemesManager
from pyams_content.shared.view import IWfView
from pyams_content.shared.view.interfaces.settings import IViewCollectionsSettings, IViewTagsSettings, \
    IViewThemesSettings
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_thesaurus.zmi.widget import ThesaurusTermsTreeFieldWidget
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminEditForm
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IPropertiesMenu
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem

__docformat__ = 'restructuredtext'

from pyams_content import _


#
# Tags management
#

@viewlet_config(name='tags.menu',
                context=IWfView, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=350,
                permission=VIEW_SYSTEM_PERMISSION)
class ViewTagsMenu(NavigationMenuItem):
    """View tags menu"""

    def __new__(cls, context, request, view, manager):
        tags_manager = ITagsManager(request.root, None)
        if (tags_manager is None) or not tags_manager.thesaurus_name:
            return None
        return NavigationMenuItem.__new__(cls)

    label = _("Tags")
    href = '#tags.html'


@ajax_form_config(name='tags.html',
                  context=IWfView, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class ViewTagsEditForm(AdminEditForm):
    """View tags settings edit form"""

    title = _("Tags settings")
    legend = _("View tags settings")

    label_css_class = 'control-label col-md-1'
    input_css_class = 'col-md-11'

    fields = Fields(IViewTagsSettings)
    fields['tags'].widget_factory = ThesaurusTermsTreeFieldWidget

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        widget = self.widgets.get('tags')
        if widget is not None:
            manager = ITagsManager(self.request.root, None)
            if manager is not None:
                widget.label_css_class = 'control-label col-md-2'
                widget.input_css_class = 'col-md-12'
                widget.thesaurus_name = manager.thesaurus_name
                widget.extract_name = manager.extract_name


#
# Themes management
#

@viewlet_config(name='themes.menu',
                context=IWfView, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=360,
                permission=VIEW_SYSTEM_PERMISSION)
class ViewThemesMenu(NavigationMenuItem):
    """View themes menu"""

    def __new__(cls, context, request, view, manager):
        themes_manager = IThemesManager(request.root, None)
        if (themes_manager is None) or not themes_manager.thesaurus_name:
            return None
        return NavigationMenuItem.__new__(cls)

    label = _("Themes")
    href = '#themes.html'


@ajax_form_config(name='themes.html',
                  context=IWfView, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class ViewThemesEditForm(AdminEditForm):
    """View themes settings edit form"""

    title = _("Themes settings")
    legend = _("View themes settings")

    label_css_class = 'control-label col-md-1'
    input_css_class = 'col-md-11'

    fields = Fields(IViewThemesSettings)
    fields['themes'].widget_factory = ThesaurusTermsTreeFieldWidget

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        widget = self.widgets.get('themes')
        if widget is not None:
            manager = IThemesManager(self.request.root, None)
            if manager is not None:
                widget.label_css_class = 'control-label col-md-2'
                widget.input_css_class = 'col-md-12'
                widget.thesaurus_name = manager.thesaurus_name
                widget.extract_name = manager.extract_name


#
# Collections management
#

@viewlet_config(name='collections.menu',
                context=IWfView, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=370,
                permission=VIEW_SYSTEM_PERMISSION)
class ViewCollectionsMenu(NavigationMenuItem):
    """View collections menu"""

    def __new__(cls, context, request, view, manager):
        collections_manager = ICollectionsManager(request.root, None)
        if (collections_manager is None) or not collections_manager.thesaurus_name:
            return None
        return NavigationMenuItem.__new__(cls)

    label = _("Collections")
    href = '#collections.html'


@ajax_form_config(name='collections.html',
                  context=IWfView, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class ViewCollectionsEditForm(AdminEditForm):
    """View collections settings edit form"""

    title = _("Collections settings")
    legend = _("View collections settings")

    label_css_class = 'control-label col-md-1'
    input_css_class = 'col-md-11'

    fields = Fields(IViewCollectionsSettings)
    fields['collections'].widget_factory = ThesaurusTermsTreeFieldWidget

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        widget = self.widgets.get('collections')
        if widget is not None:
            manager = ICollectionsManager(self.request.root, None)
            if manager is not None:
                widget.label_css_class = 'control-label col-md-2'
                widget.input_css_class = 'col-md-12'
                widget.thesaurus_name = manager.thesaurus_name
                widget.extract_name = manager.extract_name



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.paragraph.interfaces.html module

This module defines interfaces of HTML paragraphs
"""

from pyams_content.component.paragraph.interfaces import IBaseParagraph, ParagraphRendererChoice
from pyams_i18n.schema import I18nHTMLField, I18nTextField


__docformat__ = 'restructuredtext'

from pyams_content import _


#
# Raw HTML code paragraph
#

RAW_PARAGRAPH_TYPE = 'raw'
RAW_PARAGRAPH_NAME = _("Raw HTML code")
RAW_PARAGRAPH_RENDERERS = 'PyAMS_content.paragraph.raw.renderers'
RAW_PARAGRAPH_ICON_CLASS = 'fas fa-code'


class IRawParagraph(IBaseParagraph):
    """Raw HTML code paragraph interface"""

    body = I18nTextField(title=_("Source code"),
                         description=_("You can use &lt;CTRL&gt;+&lt;,&gt; to change "
                                       "editor settings"),
                         required=False)

    renderer = ParagraphRendererChoice(description=_("Paragraph renderer"),
                                       renderers=RAW_PARAGRAPH_RENDERERS)


#
# Rich text HTML paragraph
#

HTML_PARAGRAPH_TYPE = 'html'
HTML_PARAGRAPH_NAME = _("Rich text")
HTML_PARAGRAPH_RENDERERS = 'PyAMS_content.paragraph.html.renderers'
HTML_PARAGRAPH_ICON_CLASS = 'fas fa-font'


class IHTMLParagraph(IBaseParagraph):
    """Rich text paragraph interface"""

    body = I18nHTMLField(title=_("Body"),
                         required=False)

    renderer = ParagraphRendererChoice(description=_("Presentation template used for this "
                                                     "paragraph"),
                                       renderers=HTML_PARAGRAPH_RENDERERS)



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.common.zmi.manager module

This module defines common management components for shared content managers.
"""

from zope.interface import Interface

from pyams_content.interfaces import IBaseContent, MANAGE_TOOL_PERMISSION
from pyams_content.shared.common import IBaseSharedTool
from pyams_content.shared.common.interfaces import ISharedTool
from pyams_content.shared.common.manager import BaseSharedTool
from pyams_content.zmi.properties import PropertiesEditForm
from pyams_form.ajax import AJAXFormRenderer, ajax_form_config
from pyams_form.field import Fields
from pyams_form.group import Group
from pyams_form.interfaces.form import IAJAXFormRenderer, IGroup
from pyams_i18n.interfaces import II18n
from pyams_i18n_views.zmi.manager import I18nManagerLanguagesEditForm
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces import IViewContextPermissionChecker
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_skin.interfaces.viewlet import IBreadcrumbItem
from pyams_skin.viewlet.breadcrumb import BreadcrumbItem
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_viewlet.manager import viewletmanager_config
from pyams_zmi.interfaces import IAdminLayer, IObjectLabel
from pyams_zmi.interfaces.viewlet import IMenuHeader, IPropertiesMenu, ISiteManagementMenu
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem


__docformat__ = 'restructuredtext'

from pyams_content import _


@adapter_config(required=(ISharedTool, IAdminLayer, Interface),
                provides=IObjectLabel)
def shared_tool_label(context, request, view):
    """Shared tool label"""
    return II18n(context).query_attribute('title', request=request)


@adapter_config(required=(ISharedTool, IAdminLayer, Interface),
                provides=IBreadcrumbItem)
class SharedToolBreadcrumb(BreadcrumbItem):
    """Shared tool breadcrumb item"""

    @property
    def label(self):
        return II18n(self.context).query_attribute('short_name', request=self.request)

    view_name = 'admin'
    css_class = 'breadcrumb-item persistent strong'


@adapter_config(required=(ISharedTool, IAdminLayer, Interface, ISiteManagementMenu),
                provides=IMenuHeader)
def shared_tool_menu_header(context, request, view, menu):
    """Shared tool management menu header"""
    return request.localizer.translate(_("Shared content management"))


@viewletmanager_config(name='properties.menu',
                       context=ISharedTool, layer=IAdminLayer,
                       manager=ISiteManagementMenu, weight=10,
                       provides=IPropertiesMenu,
                       permission=VIEW_SYSTEM_PERMISSION)
class SharedToolPropertiesMenu(NavigationMenuItem):
    """Shared tool properties menu"""

    label = _("Properties")
    icon_class = 'fas fa-edit'
    href = '#properties.html'


@ajax_form_config(name='properties.html',
                  context=IBaseSharedTool, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class BaseSharedToolPropertiesEditForm(PropertiesEditForm):
    """Base shared tool properties edit form"""

    title = _("Shared tool properties")
    legend = _("Main tool properties")

    fields = Fields(IBaseSharedTool).omit('__name__', '__parent__')


@adapter_config(name='labels',
                required=(ISharedTool, IAdminLayer, BaseSharedToolPropertiesEditForm),
                provides=IGroup)
class SharedToolLabelsGroup(Group):
    """Shared tool properties edit form"""

    legend = _("Contents labels")

    fields = Fields(ISharedTool).select('label', 'navigation_label',
                                        'facets_label', 'facets_type_label',
                                        'dashboard_label')


@adapter_config(required=(IBaseSharedTool, IAdminLayer, BaseSharedToolPropertiesEditForm),
                provides=IAJAXFormRenderer)
class SharedToolPropertiesEditFormRenderer(AJAXFormRenderer):
    """Shared tool properties edit form renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if changes is None:
            return None
        if 'title' in changes.get(IBaseContent, ()):
            return {
                'status': 'reload',
                'message': self.request.localizer.translate(self.form.success_message)
            }
        return super().render(changes)


@adapter_config(required=(BaseSharedTool, IPyAMSLayer, I18nManagerLanguagesEditForm),
                provides=IViewContextPermissionChecker)
class SharedToolLanguagesEditFormPermissionChecker(ContextRequestViewAdapter):
    """Shared tool languages edit form permission checker"""

    edit_permission = MANAGE_TOOL_PERMISSION



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.paragraph.portlet.skin module

This module defines paragraph container portlet renderer.
"""

from zope.interface import Interface

from pyams_content.component.paragraph.interfaces import IParagraphContainer
from pyams_content.component.paragraph.portlet import IParagraphNavigationPortletSettings
from pyams_content.component.paragraph.portlet.interfaces import \
    IParagraphContainerPortletSettings
from pyams_content.component.paragraph.portlet.skin.interfaces import \
    IParagraphContainerPortletRenderer
from pyams_content.feature.renderer.interfaces import ISharedContentRenderer
from pyams_content.shared.site.interfaces import ISiteContainer
from pyams_content.skin.interfaces import IContentSummaryInfo
from pyams_layer.interfaces import IPyAMSLayer
from pyams_portal.interfaces import IPortalContext, IPortletRenderer
from pyams_portal.skin import PortletRenderer
from pyams_sequence.interfaces import IInternalReference, ISequentialIdInfo
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_utils.list import is_not_none


__docformat__ = 'restructuredtext'

from pyams_content import _


@adapter_config(required=(IPortalContext, IPyAMSLayer, Interface,
                          IParagraphContainerPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/container-default.pt', layer=IPyAMSLayer)
class ParagraphsContainerPortletRenderer(PortletRenderer):
    """Paragraphs container portlet renderer

    By default, this renderer is applying the renderer selected on each of its
    context container paragraphs.

    But you can define a custom renderer providing *IParagraphContainerPortletRenderer*,
    which will be registered with a view name; if used, and if this view name is
    matching the current request view name, this renderer will be used instead of the default
    one.

    For an example, look at form shared content submit rendering...
    """

    label = _("Paragraphs list (default)")
    weight = 1

    renderers = ()
    custom_renderer = None

    template_name = ''

    def update(self):
        super().update()
        settings = self.settings
        if settings.reference:
            source = settings.target
        else:
            source = self.context
        registry = self.request.registry
        renderer = self.custom_renderer = registry.queryMultiAdapter(
            (source, self.request, self.view), IParagraphContainerPortletRenderer,
            name=self.request.view_name)
        if renderer is not None:
            self.renderers = [renderer]
        else:
            container = IParagraphContainer(source, None)
            if container is not None:
                paragraphs = container.get_visible_paragraphs(
                    names=settings.paragraphs,
                    anchors_only=settings.anchors_only,
                    exclude_anchors=settings.exclude_anchors,
                    factories=settings.factories,
                    limit=settings.limit)
                renderers = [
                    paragraph.get_renderer(self.request)
                    for paragraph in paragraphs
                ]
            else:
                renderers = [
                    adapter
                    for name, adapter in sorted(
                        registry.getAdapters((source, self.request),
                                             ISharedContentRenderer),
                        key=lambda x: x[1].weight)
                ]
            self.renderers = tuple(filter(is_not_none, renderers))
        [renderer.update() for renderer in self.renderers]

    @property
    def use_portlets_cache(self):
        """Portlets cache checker"""
        use_cache = super().use_portlets_cache
        if use_cache and (self.custom_renderer is not None):
            return self.custom_renderer.use_portlets_cache
        return use_cache

    def get_cache_key(self):
        """Cache key getter"""
        key = super().get_cache_key()
        if self.custom_renderer is not None:
            key = f'{key}::{self.request.view_name}'
        return key

    def get_navigation_links(self):
        """Navigation links getter"""

        def test_item(nav_item):
            """Navigation item test"""
            item_sequence = ISequentialIdInfo(nav_item, None)
            if (item_sequence is not None) and (item_sequence.oid == context_sequence.oid):
                return True
            if IInternalReference.providedBy(nav_item) and \
                    (nav_item.reference == context_sequence.hex_oid):
                return True
            return False

        prev_nav, next_nav = None, None
        context_sequence = ISequentialIdInfo(self.context, None)
        if context_sequence is not None:
            display_context = self.request.display_context
            if ISiteContainer.providedBy(display_context):
                registry = self.request.registry
                previous_item = None
                items = display_context.get_visible_items(self.request)
                for item in items:
                    if test_item(item):
                        prev_nav = registry.queryMultiAdapter((previous_item, self.request),
                                                              IContentSummaryInfo)
                        break
                    previous_item = item
                try:
                    next_item = next(items)
                except (StopIteration, RuntimeError):
                    pass
                else:
                    next_nav = registry.queryMultiAdapter((next_item, self.request),
                                                          IContentSummaryInfo)
        return prev_nav, next_nav


@adapter_config(required=(IPortalContext, IPyAMSLayer, Interface,
                          IParagraphNavigationPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/navigation-default.pt', layer=IPyAMSLayer)
class ParagraphsNavigationPortletRenderer(PortletRenderer):
    """Paragraphs navigation portlet renderer

    By default, this renderer is applying the renderer selected on each of its
    context container paragraphs.

    But you can define a custom renderer providing *IParagraphContainerPortletRenderer*,
    which will be registered with a view name; if used, and if this view name is
    matching the current request view name, this renderer will be used instead of the default
    one.

    For an example, look at form shared content submit rendering...
    """

    label = _("Paragraphs navigation (default)")
    weight = 1

    paragraphs = ()

    def update(self):
        super().update()
        container = IParagraphContainer(self.context, None)
        if container is not None:
            settings = self.settings
            self.paragraphs = container.get_visible_paragraphs(
                names=settings.paragraphs,
                anchors_only=settings.anchors_only,
                factories=settings.factories)
